"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
/**
 * @module browser
 */
/** End Typedoc Module Declaration */
const core_1 = require('@angular/core');
const http_1 = require('@angular/http');
const store_1 = require('../../common/stores/store');
const logger_service_1 = require('../../common/services/logger.service');
const collection_1 = require('../../common/models/collection');
require('rxjs/add/operator/toPromise');
/**
 * HttpStore store should be extended with a specific implementation for a model. Interacts with
 * the backend over the REST API using Angular's Http service
 */
let HttpStore = class HttpStore extends store_1.AbstractStore {
    constructor(modelStatic, injector, http, loggerBase) {
        super(modelStatic, injector);
        this.http = http;
        this.logger = loggerBase.source('HTTP Store');
    }
    /**
     * Get the rest endpoint for the model
     * @param id
     * @returns {string}
     */
    endpoint(id) {
        let endpoint = `${process.env.API_BASE}/${this.modelStatic.getMetadata().storageKey}`;
        if (id) {
            endpoint += `/${id}`;
        }
        return endpoint;
    }
    ;
    /**
     * @inheritdoc
     */
    findOne(id) {
        return this.http.get(this.endpoint(id))
            .toPromise()
            .then((res) => this.checkStatus(res))
            .then((res) => this.extractModel(res))
            .catch((error) => this.handleError(error));
    }
    /**
     * @inheritdoc
     */
    findMany(query) {
        return this.http.get(this.endpoint())
            .toPromise()
            .then((res) => this.checkStatus(res))
            .then((res) => this.extractCollection(res))
            .catch((error) => this.handleError(error));
    }
    /**
     * @inheritdoc
     */
    saveOne(model) {
        //@todo consider toJson method if custom serializing is needed?
        //@todo extract only changed properties
        //@todo switch on if existing and decide if put or patch request
        return this.http.put(this.endpoint(model.getIdentifier()), model)
            .toPromise()
            .then((res) => this.checkStatus(res))
            .then(() => model); //@todo flag model as existing
        // .catch((error) => this.handleError(error));
    }
    /**
     * @inheritdoc
     */
    deleteOne(model) {
        return this.http.delete(this.endpoint(model.getIdentifier()))
            .toPromise()
            .then((res) => this.checkStatus(res))
            .then(() => model); //@todo flag model as existing
    }
    /**
     * @inheritdoc
     */
    hasOne(model) {
        return this.http.head(this.endpoint(model.getIdentifier()))
            .toPromise()
            .then((res) => this.checkStatus(res))
            .then(() => true)
            .catch(() => false);
    }
    /**
     * Extract model from the payload
     * @param res
     * @returns {T}
     */
    extractModel(res) {
        let body = res.json();
        return new this.modelStatic(body);
    }
    /**
     * Extract collection of models from the payload
     * @param res
     * @returns {Collection<T>}
     */
    extractCollection(res) {
        let body = res.json();
        return new collection_1.Collection(body.map((modelData) => new this.modelStatic(modelData)));
    }
    /**
     * Handle any exceptions
     * @param error
     * @returns {Promise<void>|Promise<T>}
     */
    handleError(error) {
        let message;
        if (error instanceof http_1.Response) {
            message = error.json();
            if (message.message) {
                message = message.message;
            }
        }
        else {
            message = error.message;
        }
        let errMsg = (message) ? message : error.status ? `${error.status} - ${error.statusText}` : 'Server error';
        this.logger.error(errMsg);
        return Promise.reject(errMsg);
    }
    /**
     * Check the status is ok.
     * This only seems to be required for unit testing @todo resolve why there is a discrepancy
     * @param res
     * @returns {any}
     */
    checkStatus(res) {
        if (res.ok) {
            return res;
        }
        return Promise.reject(res);
    }
};
HttpStore = __decorate([
    core_1.Injectable(), 
    __metadata('design:paramtypes', [Object, core_1.Injector, http_1.Http, logger_service_1.Logger])
], HttpStore);
exports.HttpStore = HttpStore;
//# sourceMappingURL=http.store.js.map