"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
const testing_1 = require('@angular/core/testing');
const testing_2 = require('@angular/http/testing');
const http_1 = require('@angular/http');
const core_1 = require('@angular/core');
const logger_service_1 = require('../../common/services/logger.service');
const logger_service_mock_1 = require('../../common/services/logger.service.mock');
const http_store_1 = require('./http.store');
const model_1 = require('../../common/models/model');
const collection_1 = require('../../common/models/collection');
const decorators_1 = require('../../common/registry/decorators');
let TestModel = class TestModel extends model_1.AbstractModel {
};
TestModel = __decorate([
    decorators_1.Model('tests'), 
    __metadata('design:paramtypes', [])
], TestModel);
let TestHttpStore = class TestHttpStore extends http_store_1.HttpStore {
    constructor(injector, http, loggerBase) {
        super(TestModel, injector, http, loggerBase);
    }
};
TestHttpStore = __decorate([
    core_1.Injectable(), 
    __metadata('design:paramtypes', [core_1.Injector, http_1.Http, logger_service_1.Logger])
], TestHttpStore);
const providers = [
    TestHttpStore,
    testing_2.MockBackend,
    http_1.BaseRequestOptions,
    {
        provide: http_1.Http,
        deps: [testing_2.MockBackend, http_1.BaseRequestOptions],
        useFactory: (backend, defaultOptions) => new http_1.Http(backend, defaultOptions),
    },
    { provide: logger_service_1.Logger, useClass: logger_service_mock_1.LoggerMock },
];
describe('Http store', () => {
    beforeEach(() => {
        testing_1.TestBed.configureTestingModule({
            providers
        });
    });
    it('Retrieves a collection of models from http', testing_1.async(testing_1.inject([TestHttpStore, testing_2.MockBackend], (s, b) => {
        let connection;
        b.connections.subscribe((c) => connection = c);
        const testPromise = s.findMany()
            .then((res) => {
            expect(res instanceof collection_1.Collection)
                .toBe(true);
            expect(res[0] instanceof TestModel)
                .toBe(true);
            expect(res[0].id)
                .toEqual(1);
            expect(res[0].name)
                .toEqual('foo');
        });
        connection.mockRespond(new http_1.Response({
            body: [{ id: 1, name: 'foo' }],
            status: 200,
            headers: null,
            url: `${process.env.API_BASE}/tests`,
            merge: null
        }));
        return testPromise;
    })));
    it('Logs error on failed collection retrieval', testing_1.async(testing_1.inject([TestHttpStore, testing_2.MockBackend], (s, b) => {
        let logSpy = spyOn(s.logger, 'error');
        let connection;
        b.connections.subscribe((c) => connection = c);
        const testPromise = s.findMany()
            .catch((res) => {
            expect(logSpy)
                .toHaveBeenCalledWith('not found');
        });
        let response = new http_1.Response({
            body: { message: 'not found' },
            status: 404,
            headers: null,
            url: `${process.env.API_BASE}/tests`,
            merge: null
        });
        connection.mockRespond(response);
        return testPromise;
    })));
    it('Retrieves a single model with http', testing_1.async(testing_1.inject([TestHttpStore, testing_2.MockBackend], (s, b) => {
        let connection;
        b.connections.subscribe((c) => connection = c);
        const testPromise = s.findOne(123)
            .then((res) => {
            expect(res instanceof TestModel)
                .toBe(true);
            expect(res.id)
                .toEqual(123);
            expect(res.name)
                .toEqual('foo');
        });
        connection.mockRespond(new http_1.Response({
            body: { id: 123, name: 'foo' },
            status: 200,
            headers: null,
            url: `${process.env.API_BASE}/tests/123`,
            merge: null
        }));
        return testPromise;
    })));
    it('Checks if a single model exists with http', testing_1.async(testing_1.inject([TestHttpStore, testing_2.MockBackend], (s, b) => {
        let connection;
        b.connections.subscribe((c) => connection = c);
        const model = new TestModel({ id: 321 });
        const testPromise = s.hasOne(model)
            .then((res) => {
            expect(res)
                .toBe(true);
            // make next request for failure, using 404 not found this time
            const promise = s.hasOne(model);
            connection.mockRespond(new http_1.Response(new http_1.ResponseOptions({
                status: 404,
            })));
            return promise;
        }).then((res) => {
            expect(res).toBe(false);
        });
        connection.mockRespond(new http_1.Response(new http_1.ResponseOptions({
            status: 204,
        })));
        return testPromise;
    })));
    it('Deletes a single model with http', testing_1.async(testing_1.inject([TestHttpStore, testing_2.MockBackend], (s, b) => {
        let connection;
        b.connections.subscribe((c) => connection = c);
        const model = new TestModel({ id: 321, name: 'delete-me' });
        const testPromise = s.deleteOne(model)
            .then((res) => {
            expect(res instanceof TestModel)
                .toBe(true);
            expect(res)
                .toEqual(model);
        });
        connection.mockRespond(new http_1.Response(new http_1.ResponseOptions({
            status: 204,
            url: `${process.env.API_BASE}/tests/123`,
        })));
        return testPromise;
    })));
    it('Logs error on failed model retrieval', testing_1.async(testing_1.inject([TestHttpStore, testing_2.MockBackend], (s, b) => {
        let logSpy = spyOn(s.logger, 'error');
        let connection;
        b.connections.subscribe((c) => connection = c);
        const testPromise = s.findOne(null)
            .catch((res) => {
            expect(logSpy)
                .toHaveBeenCalledWith('Internal Error');
        });
        connection.mockError(new Error('Internal Error'));
        return testPromise;
    })));
    it('Saves a single model with http', testing_1.async(testing_1.inject([TestHttpStore, testing_2.MockBackend], (s, b) => {
        let connection;
        b.connections.subscribe((c) => connection = c);
        const modelData = { id: 123, name: 'foo' };
        const mock = new TestModel(modelData);
        const testPromise = s.saveOne(mock)
            .then((res) => {
            expect(connection.request.json())
                .toEqual(jasmine.objectContaining(modelData));
            expect(res instanceof TestModel)
                .toBe(true);
            expect(res.id)
                .toEqual(123);
            expect(res.name)
                .toEqual('foo');
        });
        connection.mockRespond(new http_1.Response({
            body: { id: 123, name: 'foo' },
            status: 200,
            headers: null,
            url: `${process.env.API_BASE}/tests/123`,
            merge: null
        }));
        return testPromise;
    })));
});
//# sourceMappingURL=http.store.spec.js.map