/**
 * @module common
 */
/** End Typedoc Module Declaration */
import { ValidationError } from 'class-validator';
export declare abstract class HttpException extends Error {
    message: string;
    protected statusCode: number;
    protected data: any;
    constructor(message: string);
    getStatusCode(): number;
    getData(): any;
}
/**
 * Defines Error class for Bad Request errors, with HTTP status code 400
 */
export declare class BadRequestException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Unauthorized errors, with HTTP status code 401
 */
export declare class UnauthorizedException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Payment Required errors, with HTTP status code 402
 */
export declare class PaymentRequiredException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Forbidden errors, with HTTP status code 403
 */
export declare class ForbiddenException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Not Found errors, with HTTP status code 404
 */
export declare class NotFoundException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Method Not Allowed errors, with HTTP status code 405
 */
export declare class MethodNotAllowedException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Not Acceptable errors, with HTTP status code 406
 */
export declare class NotAcceptableException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Proxy Authentication Required errors, with HTTP status code 407
 */
export declare class ProxyAuthenticationRequiredException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Request Timeout errors, with HTTP status code 408
 */
export declare class RequestTimeoutException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Conflict errors, with HTTP status code 409
 */
export declare class ConflictException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Gone errors, with HTTP status code 410
 */
export declare class GoneException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Length Required errors, with HTTP status code 411
 */
export declare class LengthRequiredException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Precondition Failed errors, with HTTP status code 412
 */
export declare class PreconditionFailedException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Payload Too Large errors, with HTTP status code 413
 */
export declare class PayloadTooLargeException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for URI Too Long errors, with HTTP status code 414
 */
export declare class URITooLongException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Unsupported Media Type errors, with HTTP status code 415
 */
export declare class UnsupportedMediaTypeException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Range Not Satisfiable errors, with HTTP status code 416
 */
export declare class RangeNotSatisfiableException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Expectation Failed errors, with HTTP status code 417
 */
export declare class ExpectationFailedException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Unprocessable Entity errors, with HTTP status code 422
 */
export declare class UnprocessableEntityException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Too Many Requests errors, with HTTP status code 429
 */
export declare class TooManyRequestsException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Unavailable For Legal Reasons errors, with HTTP status code 451
 */
export declare class UnavailableForLegalReasonsException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Internal Server Error errors, with HTTP status code 500
 */
export declare class InternalServerErrorException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Not Implemented errors, with HTTP status code 501
 */
export declare class NotImplementedException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Service Unavailable errors, with HTTP status code 503
 */
export declare class ServiceUnavailableException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Insufficient Storage errors, with HTTP status code 503
 */
export declare class InsufficientStorageException extends HttpException {
    protected statusCode: number;
    constructor(message?: string);
}
/**
 * Defines Error class for Validation exceptions
 */
export declare class ValidationException extends UnprocessableEntityException {
    constructor(message: string, errors: ValidationError[]);
}
