"use strict";
const exceptions_1 = require('./exceptions');
describe('Exceptions', () => {
    const exceptions = [
        { exception: exceptions_1.BadRequestException, code: 400 },
        { exception: exceptions_1.UnauthorizedException, code: 401 },
        { exception: exceptions_1.PaymentRequiredException, code: 402 },
        { exception: exceptions_1.ForbiddenException, code: 403 },
        { exception: exceptions_1.NotFoundException, code: 404 },
        { exception: exceptions_1.MethodNotAllowedException, code: 405 },
        { exception: exceptions_1.NotAcceptableException, code: 406 },
        { exception: exceptions_1.ProxyAuthenticationRequiredException, code: 407 },
        { exception: exceptions_1.RequestTimeoutException, code: 408 },
        { exception: exceptions_1.ConflictException, code: 409 },
        { exception: exceptions_1.GoneException, code: 410 },
        { exception: exceptions_1.LengthRequiredException, code: 411 },
        { exception: exceptions_1.PreconditionFailedException, code: 412 },
        { exception: exceptions_1.PayloadTooLargeException, code: 413 },
        { exception: exceptions_1.URITooLongException, code: 414 },
        { exception: exceptions_1.UnsupportedMediaTypeException, code: 415 },
        { exception: exceptions_1.RangeNotSatisfiableException, code: 416 },
        { exception: exceptions_1.ExpectationFailedException, code: 417 },
        { exception: exceptions_1.UnprocessableEntityException, code: 422 },
        { exception: exceptions_1.TooManyRequestsException, code: 429 },
        { exception: exceptions_1.UnavailableForLegalReasonsException, code: 451 },
        { exception: exceptions_1.InternalServerErrorException, code: 500 },
        { exception: exceptions_1.NotImplementedException, code: 501 },
        { exception: exceptions_1.ServiceUnavailableException, code: 503 },
        { exception: exceptions_1.InsufficientStorageException, code: 507 },
    ];
    exceptions.forEach((check) => {
        it(`creates instance of ${check.exception.prototype.constructor.name} with status code ${check.code}`, () => {
            const exceptionInstance = new check.exception;
            expect(exceptionInstance instanceof exceptions_1.HttpException).toBe(true, 'instanceof HttpException');
            expect(exceptionInstance instanceof Error).toBe(true, 'instanceof Error');
            expect(exceptionInstance.getStatusCode()).toEqual(check.code);
            expect(exceptionInstance.name).toEqual(exceptionInstance.constructor.name);
        });
    });
    it('creates an exception that is an instance of it\'s parents', () => {
        let exception = new exceptions_1.ValidationException(null, []);
        expect(exception instanceof exceptions_1.HttpException).toBe(true);
        expect(exception instanceof exceptions_1.UnprocessableEntityException).toBe(true);
    });
    it('retrieves data from exception with .data()', () => {
        let errors = [{
                target: null,
                property: 'name',
                constraints: {
                    max_length: 'too long',
                },
                value: 10,
                children: undefined,
            }];
        let exception = new exceptions_1.ValidationException(null, errors);
        expect(exception.getData()).toEqual(errors);
    });
});
//# sourceMappingURL=exceptions.spec.js.map