"use strict";
/**
 * Collection holds an array of [[AbstractModel|models]]. It provides common collection manipulation
 * methods for the controllers, services etc to work with models in an abstracted manner
 */
class Collection extends Array {
    constructor(initialItems) {
        super();
        if (initialItems.length) {
            this.push.apply(this, initialItems);
        }
    }
    /**
     * Find an item in the collection by primary identifier
     * @param id
     * @returns {T}
     */
    findById(id) {
        const found = this.find((model) => model.getIdentifier() === id);
        if (!found) {
            throw new Error(`Item with id [${id}] not in collection`);
        }
        return found;
    }
    /**
     * Remove an item from the collection
     * @param model
     */
    remove(model) {
        const index = this.findIndex((item) => item.getIdentifier() === model.getIdentifier());
        if (index >= 0) {
            this.splice(index, 1);
        }
    }
    /**
     * Check if the collection contains a given model
     * @param model
     * @returns {boolean}
     */
    contains(model) {
        try {
            this.findById(model.getIdentifier());
            return true;
        }
        catch (e) {
        }
        return false;
    }
}
exports.Collection = Collection;
//# sourceMappingURL=collection.js.map