"use strict";
/**
 * @module common
 */
/** End Typedoc Module Declaration */
const entityRegistry_1 = require('../registry/entityRegistry');
/**
 * Helper class for differentiating string keys with uuid keys
 */
class UUID extends String {
    constructor(value) {
        super(value);
    }
}
exports.UUID = UUID;
/**
 * Common abstract class that **all** models must extend from. Provides common interfaces for other
 * services to interact with without knowing about the concrete implementation
 */
class AbstractModel extends entityRegistry_1.RegistryEntity {
    constructor(data) {
        super();
        this.hydrate(data);
    }
    /**
     * Hydrates the model from given data
     * @param data
     * @returns {AbstractModel}
     */
    hydrate(data) {
        if (!data) {
            return this;
        }
        Object.assign(this, data);
        return this;
    }
    /**
     * Get the primary identifier of the model
     * @returns {any}
     */
    getIdentifier() {
        return this[this.getMetadata().identifierKey];
    }
}
exports.AbstractModel = AbstractModel;
//# sourceMappingURL=model.js.map