/**
 * @module common
 */
/** End Typedoc Module Declaration */
import { AbstractModel } from '../model';
import { ForeignRelationModelGetter, ViaPropertyDefinition } from './index';
import { RelationOptions } from 'typeorm/decorator/options/RelationOptions';
/**
 * Defines the relationship between the current model and a foreign model via the decorated key
 *
 * Example:
 * ```typescript
 *
 *  @Model
 *  class Thumb extends AbstractModel {
 *
 *    @BelongsTo(f => HandModel, hand => hand.handId)
 *    public hand: HandModel;
 *
 *  }
 *
 * ```
 */
export declare function BelongsTo<M extends AbstractModel, F extends AbstractModel>(foreignTypeGetter: ForeignRelationModelGetter<M, F>, viaProperty: ViaPropertyDefinition<F>, joinOptions?: RelationOptions): PropertyDecorator;
