"use strict";
const index_1 = require('./index');
/**
 * Defines the relationship between the current model and a foreign model via the decorated key
 *
 * Example:
 * ```typescript
 *
 *  @Model
 *  class Thumb extends AbstractModel {
 *
 *    @BelongsTo(f => HandModel, hand => hand.handId)
 *    public hand: HandModel;
 *
 *  }
 *
 * ```
 */
function BelongsTo(foreignTypeGetter, viaProperty, joinOptions) {
    return (target, propertyKey) => {
        index_1.initializeRelationMap(target, 'belongsTo');
        target.constructor.__metadata.relations.get('belongsTo')
            .set(propertyKey, new index_1.Relation(target.constructor, foreignTypeGetter, viaProperty, joinOptions));
    };
}
exports.BelongsTo = BelongsTo;
//# sourceMappingURL=belongsTo.decorator.js.map