/**
 * @module common
 */
/** End Typedoc Module Declaration */
import { AbstractModel } from '../model';
import { ForeignRelationModelGetter } from './index';
import { RelationOptions } from 'typeorm/decorator/options/RelationOptions';
/**
 * Defines the relationship between the current model and a foreign model via the decorated key
 *
 * Example:
 * ```typescript
 *
 *  @Model
 *  class Hand extends AbstractModel {
 *
 *    @HasOne(f => ThumbModel)
 *    public thumb: ThumbModel;
 *  }
 *
 * ```
 */
export declare function HasOne<M extends AbstractModel, F extends AbstractModel>(foreignTypeGetter: ForeignRelationModelGetter<M, F>, joinOptions?: RelationOptions): PropertyDecorator;
