"use strict";
const index_1 = require('./index');
/**
 * Defines the relationship between the current model and a foreign model via the decorated key
 *
 * Example:
 * ```typescript
 *
 *  @Model
 *  class Hand extends AbstractModel {
 *
 *    @HasOne(f => ThumbModel)
 *    public thumb: ThumbModel;
 *  }
 *
 * ```
 */
function HasOne(foreignTypeGetter, joinOptions) {
    return (target, propertyKey) => {
        index_1.initializeRelationMap(target, 'hasOne');
        target.constructor.__metadata.relations.get('hasOne')
            .set(propertyKey, new index_1.Relation(target.constructor, foreignTypeGetter, null, joinOptions));
    };
}
exports.HasOne = HasOne;
//# sourceMappingURL=hasOne.decorator.js.map