/**
 * @module common
 */
/** End Typedoc Module Declaration */
import { ModelStatic, ModelConstructor, AbstractModel } from '../model';
export declare type RelationType = 'hasOne' | 'hasMany' | 'belongsTo' | 'belongsToMany';
/**
 * This is a crude method to two-way register the type of binding for relations. This is to overcome
 * a limitation of Typescripts design-time decorators and node's module resolution.
 * @see https://github.com/Microsoft/TypeScript/issues/4521
 */
export declare type ForeignRelationModelGetter<T extends AbstractModel, F extends AbstractModel> = (thisStatic?: ModelStatic<T> | any) => ModelStatic<F>;
export declare type ViaPropertyDefinition<T> = (foreign: T) => any;
export declare class Relation<M extends AbstractModel, F extends AbstractModel> {
    model: ModelStatic<M>;
    private foreignRelationModelGetter;
    viaProperty: ViaPropertyDefinition<F>;
    databaseOptions: any;
    constructor(model: ModelStatic<M>, foreignRelationModelGetter: ForeignRelationModelGetter<M, F>, viaProperty: ViaPropertyDefinition<F>, databaseOptions?: any);
    readonly foreign: ModelStatic<F>;
}
/**
 * Initializes relation metadata property with empty values. Common function used by all relation
 * decorators to verify there is somewhere to assign their metadata.
 * @param target
 * @param type
 */
export declare function initializeRelationMap(target: ModelConstructor<any>, type: RelationType): void;
export * from './hasOne.decorator';
export * from './belongsTo.decorator';
