/**
 * @module common
 */
/** End Typedoc Module Declaration */
import { EntityType, EntityMetadata } from './entityRegistry';
import { ModelMetadata } from '../metadata/metadata';
/**
 * Common decorator factory function to simplify decorator declarations
 * @param type
 * @param metadata
 * @returns {function(TFunction): void}
 */
export declare function entityRegistryFunction(type: EntityType, metadata?: EntityMetadata): ClassDecorator;
/**
 *  @Model decorator for registering class with the [[EntityRegistry]]
 *
 * class Example:
 * ```typescript
 *
 *  @Model()
 *  export class ExampleModel extends AbstractModel {}
 *  import { Model, AbstractModel } from '@zerothstack/core/common';
 * ```
 * @param metadata
 * @returns {ClassDecorator}
 * @constructor
 */
export declare function Model(metadata?: ModelMetadata): ClassDecorator;
/**
 *  @Store class decorator for registering class with the [[EntityRegistry]]
 *
 * Example:
 * ```typescript
 *  import { Store, AbstractStore } from '@zerothstack/core/common';
 *
 *  @Store()
 *  export class ExampleStore extends AbstractStore<any> {}
 * ```
 * @returns {ClassDecorator}
 * @constructor
 */
export declare function Store(): ClassDecorator;
/**
 *  @Service class decorator for registering class with the [[EntityRegistry]]
 *
 * Example:
 * ```typescript
 *  import { Service, AbstractService } from '@zerothstack/core/common';
 *
 *  @Service()
 *  export class ExampleService extends AbstractService {}
 * ```
 * @returns {ClassDecorator}
 * @constructor
 */
export declare function Service(): ClassDecorator;
