"use strict";
/**
 * @module common
 */
/** End Typedoc Module Declaration */
const entityRegistry_1 = require('./entityRegistry');
/**
 * Common decorator factory function to simplify decorator declarations
 * @param type
 * @param metadata
 * @returns {function(TFunction): void}
 */
function entityRegistryFunction(type, metadata) {
    return function (target) {
        entityRegistry_1.EntityRegistry.register(type, target, metadata);
    };
}
exports.entityRegistryFunction = entityRegistryFunction;
/**
 *  @Model decorator for registering class with the [[EntityRegistry]]
 *
 * class Example:
 * ```typescript
 *
 *  @Model()
 *  export class ExampleModel extends AbstractModel {}
 *  import { Model, AbstractModel } from '@zerothstack/core/common';
 * ```
 * @param metadata
 * @returns {ClassDecorator}
 * @constructor
 */
function Model(metadata) {
    return entityRegistryFunction('model', metadata);
}
exports.Model = Model;
/**
 *  @Store class decorator for registering class with the [[EntityRegistry]]
 *
 * Example:
 * ```typescript
 *  import { Store, AbstractStore } from '@zerothstack/core/common';
 *
 *  @Store()
 *  export class ExampleStore extends AbstractStore<any> {}
 * ```
 * @returns {ClassDecorator}
 * @constructor
 */
function Store() {
    return entityRegistryFunction('store');
}
exports.Store = Store;
/**
 *  @Service class decorator for registering class with the [[EntityRegistry]]
 *
 * Example:
 * ```typescript
 *  import { Service, AbstractService } from '@zerothstack/core/common';
 *
 *  @Service()
 *  export class ExampleService extends AbstractService {}
 * ```
 * @returns {ClassDecorator}
 * @constructor
 */
function Service() {
    return entityRegistryFunction('service');
}
exports.Service = Service;
//# sourceMappingURL=decorators.js.map