"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
/**
 * @module common
 */
/** End Typedoc Module Declaration */
const logger_service_1 = require('./logger.service');
const chalk_1 = require('chalk');
const util_1 = require('util');
const core_1 = require('@angular/core');
const moment = require('moment');
const decorators_1 = require('../registry/decorators');
exports.isBrowser = () => {
    return typeof window !== 'undefined';
};
/**
 * A concrete implementation of [[Logger]] this is a generic logger that will log to the console.
 * It can be used in both frontend and backend, and will log to the respective consoles.
 * Only in the NodeJS environment does colour highlighting take place
 */
let ConsoleLogger_1 = class ConsoleLogger extends logger_service_1.Logger {
    constructor() {
        super(ConsoleLogger_1);
        this.envBrowser = exports.isBrowser();
    }
    /**
     * Format the log with an appropriate colour
     * @param logLevel
     * @param message
     * @returns {string}
     */
    format(logLevel, message) {
        switch (logLevel) {
            case 'emergency':
                message = chalk_1.bgRed(message);
                break;
            case 'alert':
                message = chalk_1.red.underline(message);
                break;
            case 'critical':
                message = chalk_1.yellow.underline(message);
                break;
            case 'warning':
                message = chalk_1.yellow(message);
                break;
            case 'notice':
                message = chalk_1.magenta(message);
                break;
            case 'info':
                message = chalk_1.blue(message);
                break;
            case 'debug':
                message = chalk_1.gray(message);
                break;
        }
        return message;
    }
    /**
     * Format the messages - in node env anything that is not a string is passed into util.inspect
     * for coloured syntax highlighting
     * @param logLevel
     * @param messages
     * @returns {any}
     */
    formatMessages(logLevel, messages) {
        // if in browser, defer to the browser for formatting
        if (this.envBrowser) {
            return messages;
        }
        return messages.map((message) => {
            switch (typeof message) {
                case 'string':
                    return this.format(logLevel, message);
                default:
                    return util_1.inspect(message, {
                        colors: true
                    });
            }
        });
    }
    /**
     * Output the log to console. The log messages are prepended with the current time and source if
     * set
     * @param logLevel
     * @param messages
     * @returns {ConsoleLogger}
     */
    persistLog(logLevel, messages) {
        messages = this.formatMessages(logLevel, messages);
        if (this.sourceName) {
            messages.unshift(chalk_1.gray('[' + this.format(logLevel, this.sourceName) + ']'));
        }
        messages.unshift(chalk_1.gray('[' + this.format(logLevel, moment()
            .format('HH:mm:ss')) + '] '));
        switch (logLevel) {
            case 'emergency':
            case 'alert':
            case 'critical':
            case 'error':
                console.error(messages.shift(), ...messages);
                break;
            case 'warning':
            case 'notice':
                console.warn(messages.shift(), ...messages);
                break;
            default:
                console.log(messages.shift(), ...messages);
        }
        return this;
    }
    ;
};
let ConsoleLogger = ConsoleLogger_1;
ConsoleLogger = ConsoleLogger_1 = __decorate([
    decorators_1.Service(),
    core_1.Injectable(), 
    __metadata('design:paramtypes', [])
], ConsoleLogger);
exports.ConsoleLogger = ConsoleLogger;
//# sourceMappingURL=consoleLogger.service.js.map