"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const collection_1 = require('../models/collection');
const store_1 = require('./store');
const chance_1 = require('chance');
const _ = require('lodash');
/**
 * Provides abstract class to build concrete mock stores which can create new mock instances of
 * models.
 */
class MockStore extends store_1.AbstractStore {
    constructor(modelStatic, injector) {
        super(modelStatic, injector);
        this.initializeMockCollection();
    }
    /**
     * Start the mock store off with some dummy data
     */
    initializeMockCollection() {
        const models = _.times(10, () => this.getMock());
        this.modelCollection = new collection_1.Collection(models);
    }
    /**
     * Retrieve instance of chance, optionally providing a seed for the internal mersenne twister to
     * get repeatable random data
     * @param seed
     * @returns {ChanceInstance}
     */
    chance(seed) {
        if (!this.chanceInstance || !!seed) {
            this.chanceInstance = new chance_1.Chance(seed);
        }
        return this.chanceInstance;
    }
    /**
     * @inheritdoc
     */
    findOne(id) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield this.modelCollection.findById(id);
            }
            catch (e) {
                return yield this.saveOne(this.getMock(id));
            }
        });
    }
    /**
     * @inheritdoc
     */
    findMany(query) {
        return Promise.resolve(this.modelCollection);
    }
    /**
     * Mock saving the model
     *
     * As saving does not make sense for a mock store, this just stubs the interface by returning
     * the model in a resolved promise
     */
    saveOne(model) {
        this.modelCollection.push(model);
        return Promise.resolve(model);
    }
    /**
     * Mock selecting model by id
     *
     * As deleting does not make sense for a mock store, this just stubs the interface by returning
     * the model in a resolved promise
     * @param model
     * @returns {Promise<void>}
     */
    deleteOne(model) {
        this.modelCollection.remove(model);
        return Promise.resolve(model);
    }
    /**
     * @inheritdoc
     */
    hasOne(model) {
        return Promise.resolve(this.modelCollection.contains(model));
    }
}
exports.MockStore = MockStore;
//# sourceMappingURL=mock.store.js.map