"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const validation_1 = require('../validation');
const exceptions_1 = require('../exceptions/exceptions');
/**
 * The abstract store should be the root calls for *all* stores, it provides common methods
 * for entity validation and storage.
 */
class AbstractStore {
    constructor(modelStatic, injector) {
        this.modelStatic = modelStatic;
        this.injector = injector;
        this.validator = validation_1.getValidator(injector);
    }
    /**
     * Promise that store is initialized.
     * Override this function for stores that have async initialization like Database stores that
     * require a connection etc.
     * @returns {Promise<AbstractStore>}
     */
    initialized() {
        return Promise.resolve(this);
    }
    // public abstract saveMany(models:Collection<T>): Promise<Collection<T>>;
    // public abstract deleteMany(models:Collection<T>): Promise<void>;
    /**
     * Check the entity passed is valid, if not throw ValidationException with the errors
     * @param model
     * @param validatorOptions
     * @returns {Promise<T>}
     */
    validate(model, validatorOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const errors = yield this.validator.validate(model, validatorOptions);
            if (errors.length) {
                throw new exceptions_1.ValidationException(null, errors);
            }
            return model;
        });
    }
    /**
     * Build the entity from data
     * @param modelData
     * @returns {Promise<T>}
     */
    hydrate(modelData) {
        return Promise.resolve(new this.modelStatic(modelData));
    }
}
exports.AbstractStore = AbstractStore;
//# sourceMappingURL=store.js.map