"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
const testing_1 = require('@angular/core/testing');
const core_1 = require('@angular/core');
const mock_store_1 = require('./mock.store');
const model_1 = require('../models/model');
const store_1 = require('./store');
const validation_1 = require('../validation');
const primary_decorator_1 = require('../models/types/primary.decorator');
const exceptions_1 = require('../exceptions/exceptions');
let StubService = class StubService {
    isTruthy(value) {
        return !!value;
    }
    isTruthyPromise(value) {
        return Promise.resolve(!!value);
    }
};
StubService = __decorate([
    core_1.Injectable(), 
    __metadata('design:paramtypes', [])
], StubService);
let CustomValidator = class CustomValidator {
    constructor(service) {
        this.service = service;
    }
    validate(value) {
        return this.service.isTruthy(value);
    }
};
CustomValidator = __decorate([
    core_1.Injectable(),
    validation_1.ValidatorConstraint(), 
    __metadata('design:paramtypes', [StubService])
], CustomValidator);
let CustomValidatorAsync = class CustomValidatorAsync {
    constructor(service) {
        this.service = service;
    }
    validate(value) {
        return this.service.isTruthyPromise(value);
    }
};
CustomValidatorAsync = __decorate([
    core_1.Injectable(),
    validation_1.ValidatorConstraint(), 
    __metadata('design:paramtypes', [StubService])
], CustomValidatorAsync);
class Ship extends model_1.AbstractModel {
}
__decorate([
    primary_decorator_1.Primary(), 
    __metadata('design:type', Number)
], Ship.prototype, "shipId", void 0);
__decorate([
    validation_1.MinLength(2), 
    __metadata('design:type', String)
], Ship.prototype, "name", void 0);
__decorate([
    validation_1.Validate(CustomValidator), 
    __metadata('design:type', Object)
], Ship.prototype, "truthyValue", void 0);
__decorate([
    validation_1.Validate(CustomValidatorAsync), 
    __metadata('design:type', Object)
], Ship.prototype, "truthyValueAsync", void 0);
class ShipStore extends store_1.AbstractStore {
}
let ShipMockStore = class ShipMockStore extends mock_store_1.MockStore {
    constructor(injector) {
        super(Ship, injector);
    }
    getMock(id) {
        return new this.modelStatic({
            shipId: id || this.chance()
                .integer(),
            name: 'Razorback',
            idRandom: this.chance()
                .guid(),
            idSeeded: this.chance(1)
                .guid(),
            truthyValue: true,
            truthyValueAsync: true,
        });
    }
};
ShipMockStore = __decorate([
    core_1.Injectable(), 
    __metadata('design:paramtypes', [core_1.Injector])
], ShipMockStore);
let TestClass = class TestClass {
    constructor(shipStore) {
        this.shipStore = shipStore;
    }
};
TestClass = __decorate([
    core_1.Injectable(), 
    __metadata('design:paramtypes', [ShipStore])
], TestClass);
//we need to have a singleton instance here to spy on, as the validator creates new singletons
// of the custom validator
const stubServiceSingleton = new StubService();
const providers = [
    TestClass,
    CustomValidator,
    CustomValidatorAsync,
    { provide: StubService, deps: [], useFactory: () => stubServiceSingleton },
    { provide: ShipStore, useClass: ShipMockStore },
];
describe('Mock Store', () => {
    beforeEach(() => {
        testing_1.TestBed.configureTestingModule({ providers });
    });
    it('is injected with the store instance token', testing_1.inject([TestClass], (c) => {
        expect(c instanceof TestClass)
            .toBe(true);
        expect(c.shipStore instanceof store_1.AbstractStore)
            .toBe(true);
        expect(c.shipStore instanceof ShipMockStore)
            .toBe(true);
    }));
    it('is provides a dummy initialized method', testing_1.inject([TestClass], (c) => {
        c.shipStore.initialized()
            .then((instance) => {
            expect(instance)
                .toEqual(c.shipStore);
        });
    }));
    it('retrieves a promise of a mock entity', testing_1.async(testing_1.inject([TestClass], (c) => {
        return c.shipStore.findOne(1234)
            .then((entity) => {
            expect(entity instanceof Ship)
                .toBe(true);
            expect(entity.getIdentifier())
                .toBe(1234);
        });
    })));
    it('mocks random and seeded random data', testing_1.async(testing_1.inject([TestClass], (c) => {
        return c.shipStore.findMany(1234)
            .then((ships) => {
            expect(ships[0].idRandom)
                .not
                .toEqual(ships[1].idRandom);
            expect(ships[0].idSeeded)
                .toEqual(ships[1].idSeeded);
        });
    })));
    it('mocks save action', testing_1.async(testing_1.inject([TestClass], (c) => {
        let shipRef = null;
        return c.shipStore.findOne(1234)
            .then((ship) => {
            shipRef = ship;
            return c.shipStore.saveOne(ship);
        })
            .then((ship) => {
            expect(shipRef)
                .toEqual(ship);
            return c.shipStore.findMany();
        })
            .then((allModels) => {
            expect(allModels.findById(1234)).toEqual(shipRef);
        });
    })));
    it('mocks delete action', testing_1.async(testing_1.inject([TestClass], (c) => {
        let shipRef = null;
        return c.shipStore.findOne(1234)
            .then((ship) => {
            shipRef = ship;
            return c.shipStore.deleteOne(ship);
        })
            .then((ship) => {
            expect(ship instanceof Ship)
                .toBe(true);
            return c.shipStore.findMany();
        })
            .then((allModels) => {
            expect(() => allModels.findById(1234)).toThrowError('Item with id [1234] not in collection');
        });
    })));
    it('explicitly hydrates from raw data', testing_1.async(testing_1.inject([TestClass], (c) => {
        let data = {
            shipId: 321,
            name: 'Rocinante'
        };
        return c.shipStore.hydrate(data)
            .then((ship) => {
            expect(ship instanceof Ship)
                .toBe(true);
            expect(ship.getIdentifier())
                .toEqual(321);
            expect(ship.name)
                .toEqual(data.name);
        });
    })));
    it('validates model data, returning instance on pass', testing_1.async(testing_1.inject([TestClass], (c) => {
        let shipRef = null;
        return c.shipStore.findOne(1234)
            .then((ship) => {
            shipRef = ship;
            return c.shipStore.validate(ship);
        })
            .then((ship) => {
            expect(shipRef)
                .toEqual(ship);
        });
    })));
    it('validates model data, returning ValidationException with data on failure', testing_1.async(testing_1.inject([TestClass], (c) => {
        return c.shipStore.findOne(1234)
            .then((ship) => {
            ship.name = 'a'; //too short
            return c.shipStore.validate(ship);
        })
            .catch((e) => {
            expect(e instanceof exceptions_1.ValidationException)
                .toBe(true);
            expect(e.getData()[0].constraints.minLength).not
                .toBeNull();
        });
    })));
    it('validates model data, with custom validation with injected service dependency', testing_1.async(testing_1.inject([TestClass, StubService], (c, s) => {
        let customValidatorServiceSpy = spyOn(stubServiceSingleton, 'isTruthy')
            .and
            .callThrough();
        return c.shipStore.findOne(1234)
            .then((ship) => {
            ship.truthyValue = 'truthy';
            return c.shipStore.validate(ship);
        })
            .then((ship) => {
            expect(customValidatorServiceSpy)
                .toHaveBeenCalledWith('truthy');
        });
    })));
    it('validates model data, with custom async validator', testing_1.async(testing_1.inject([TestClass, StubService], (c, s) => {
        let customValidatorServiceSpy = spyOn(stubServiceSingleton, 'isTruthyPromise')
            .and
            .callThrough();
        return c.shipStore.findOne(1234)
            .then((ship) => {
            ship.truthyValueAsync = false;
            return c.shipStore.validate(ship);
        })
            .catch((e) => {
            expect(customValidatorServiceSpy)
                .toHaveBeenCalledWith(false);
            expect(e instanceof exceptions_1.ValidationException)
                .toBe(true);
            expect(e.getData()[0].constraints.CustomValidatorAsync)
                .not.toBeNull();
        });
    })));
});
//# sourceMappingURL=store.spec.js.map