"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
const class_validator_1 = require('class-validator');
/**
 * class-validator has a basic inversion container that handles classes without constructor arguments
 * This hybrid class re-implements that injector, and extends it to be able to use the core angular
 * injector when available.
 */
class HybridInjector {
    constructor() {
        this.instances = new WeakMap();
    }
    /**
     * Get the instance of a class when there is no dependency
     * @param staticClass
     * @returns {any}
     */
    noDependencyGet(staticClass) {
        if (!this.instances.has(staticClass)) {
            this.instances.set(staticClass, new staticClass());
        }
        return this.instances.get(staticClass);
    }
    get(staticClass) {
        try {
            if (!this.injector) {
                return this.noDependencyGet(staticClass);
            }
            return this.injector.get(staticClass);
        }
        catch (e) {
            return this.noDependencyGet(staticClass);
        }
    }
}
const hybridInjectorInstance = new HybridInjector();
class_validator_1.useContainer(hybridInjectorInstance);
/**
 * Get the (singleton) validator instance and assign the dependency injector so custom validators
 * can use the DI service.
 * @param injector
 * @returns {Validator}
 */
function getValidator(injector) {
    hybridInjectorInstance.injector = injector;
    return hybridInjectorInstance.get(class_validator_1.Validator);
}
exports.getValidator = getValidator;
// This common export is used so that both the core and the implementing modules
// use the same instance. This is necessary as class-validator does a single export
// of the MetadataStorage class which is used as a singleton. Otherwise, the models
// may register their validations against a different store than what is validated with
__export(require('class-validator'));
//# sourceMappingURL=index.js.map