"use strict";
/**
 * @module server
 */
/** End Typedoc Module Declaration */
const entity_bootstrapper_1 = require('./entity.bootstrapper');
/**
 * Provides bootstrapping of the @[[Migration]] entities
 */
class MigrationBootstrapper extends entity_bootstrapper_1.EntityBootstrapper {
    /**
     * Returns all migrations registered to the [[EntityRegistry]]
     */
    getInjectableEntities() {
        return this.getEntitiesFromRegistry('migration');
    }
    /**
     * Runs all migrations, awaiting completion of all before the main bootstrapper proceed
     * @todo update to assign tasks to the remote cli so it doesn't happen on startup
     * @returns {Promise<any>|Promise<void[]>}
     */
    bootstrap() {
        this.logger.debug(`Running [${this.entities.length}] migrations`);
        const allMigrationPromises = this.entities.map((resolvedMigration) => {
            this.logger.info(`migrating ${resolvedMigration.constructor.name}`);
            return this.getInstance(resolvedMigration)
                .migrate()
                .catch((error) => {
                if (error.code === 'ECONNREFUSED') {
                    this.logger.notice('Database not available, migration cannot run');
                    return;
                }
                throw error;
            });
        }, []);
        return Promise.all(allMigrationPromises);
    }
}
exports.MigrationBootstrapper = MigrationBootstrapper;
//# sourceMappingURL=migrations.bootstrapper.js.map