"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
const testing_1 = require('@angular/core/testing');
const core_1 = require('@angular/core');
const remoteCli_service_mock_1 = require('../services/remoteCli.service.mock');
const remoteCli_service_1 = require('../services/remoteCli.service');
const abstract_server_mock_1 = require('../servers/abstract.server.mock');
const logger_service_1 = require('../../common/services/logger.service');
const logger_service_mock_1 = require('../../common/services/logger.service.mock');
const abstract_server_1 = require('../servers/abstract.server');
const bootstrap_1 = require('./bootstrap');
const entityRegistry_1 = require('../../common/registry/entityRegistry');
const index_1 = require('../migrations/index');
const database_service_1 = require('../services/database.service');
const database_service_mock_1 = require('../services/database.service.mock');
const auth_service_mock_1 = require('../services/auth.service.mock');
const auth_service_1 = require('../services/auth.service');
let loggerInstance = new logger_service_mock_1.LoggerMock();
let databaseInstance;
const providers = [
    {
        provide: logger_service_1.Logger,
        deps: [],
        useValue: loggerInstance
    },
    {
        provide: database_service_1.Database,
        deps: [logger_service_1.Logger],
        useFactory: (logger) => {
            if (!databaseInstance) {
                databaseInstance = new database_service_mock_1.DatabaseMock(logger);
            }
            return databaseInstance;
        }
    },
    { provide: abstract_server_1.Server, useClass: abstract_server_mock_1.ServerMock },
    { provide: remoteCli_service_1.RemoteCli, useClass: remoteCli_service_mock_1.RemoteCliMock },
    { provide: auth_service_1.AuthService, useClass: auth_service_mock_1.AuthServiceMock },
];
let TestMigration = class TestMigration extends index_1.AbstractMigration {
    constructor(logger, database) {
        super(logger, database);
    }
    migrate() {
        return this.database.initialize()
            .then(() => {
            this.logger.debug('Test migration running');
            return Promise.resolve();
        });
    }
    rollback() {
    }
};
TestMigration = __decorate([
    core_1.Injectable(), 
    __metadata('design:paramtypes', [logger_service_1.Logger, database_service_1.Database])
], TestMigration);
exports.TestMigration = TestMigration;
describe('Migration Bootstrapper', () => {
    beforeEach(() => {
        testing_1.TestBed.configureTestingModule({ providers });
        entityRegistry_1.EntityRegistry.clearAll();
        entityRegistry_1.EntityRegistry.register('migration', TestMigration);
    });
    it('resolves and runs migration', (done) => {
        const loggerSpy = spyOn(loggerInstance, 'persistLog')
            .and
            .callThrough();
        spyOn(loggerInstance, 'source')
            .and
            .callFake(() => loggerInstance);
        const result = bootstrap_1.bootstrap(undefined, providers)();
        return result.then((res) => {
            expect(loggerSpy)
                .toHaveBeenCalledWith('debug', ['Test migration running']);
            done();
        });
    });
    it('continues bootstrap even when database refuses connection', (done) => {
        const loggerSpy = spyOn(loggerInstance, 'persistLog')
            .and
            .callThrough();
        spyOn(loggerInstance, 'source')
            .and
            .callFake(() => loggerInstance);
        spyOn(databaseInstance, 'initialize')
            .and
            .callFake(() => {
            const err = new Error();
            err.code = 'ECONNREFUSED';
            return Promise.reject(err);
        });
        const result = bootstrap_1.bootstrap(undefined, providers)();
        return result.then((res) => {
            expect(loggerSpy)
                .toHaveBeenCalledWith('notice', ['Database not available, migration cannot run']);
            done();
        });
    });
    it('aborts bootstrap when a fatal error occurs', (done) => {
        //as there is a fallback to output to log when a fatal bootstrap happens even when mocked,
        //here we spy on the console to suppress the log output
        spyOn(console, 'error');
        spyOn(console, 'log');
        const processExitSpy = spyOn(process, 'exit');
        const loggerSpy = spyOn(loggerInstance, 'persistLog')
            .and
            .callThrough();
        spyOn(loggerInstance, 'source')
            .and
            .callFake(() => loggerInstance);
        spyOn(databaseInstance, 'initialize')
            .and
            .callFake(() => {
            const err = new Error('Something else went wrong');
            return Promise.reject(err);
        });
        const result = bootstrap_1.bootstrap(undefined, providers)();
        return result.then((res) => {
            expect(processExitSpy)
                .toHaveBeenCalledWith(1);
            expect(loggerSpy)
                .toHaveBeenCalledWith('critical', ['Error', 'Something else went wrong']);
            done();
        });
    });
});
//# sourceMappingURL=migrations.bootstrapper.spec.js.map