"use strict";
/**
 * @module server
 */
/** End Typedoc Module Declaration */
const entity_bootstrapper_1 = require('./entity.bootstrapper');
/**
 * Provides bootstrapping of the @[[Service]] entities
 */
class ServiceBootstrapper extends entity_bootstrapper_1.EntityBootstrapper {
    /**
     * Returns all services registered to the [[EntityRegistry]]
     */
    getInjectableEntities() {
        return this.getEntitiesFromRegistry('service');
    }
    /**
     * Bootstrap all services. With each service, the initialize function is invoked, and the bootstrap
     * awaits all services to complete initializing before resolving the promise
     * @returns {Promise<any>|Promise<AbstractService[]>}
     */
    bootstrap() {
        this.logger.debug(`Initializing [${this.entities.length}] services`);
        const allServicePromises = this.entities.map((resolvedService) => {
            let service = this.getInstance(resolvedService);
            return Promise.resolve(service.initialize());
        }, []);
        return Promise.all(allServicePromises);
    }
}
exports.ServiceBootstrapper = ServiceBootstrapper;
//# sourceMappingURL=services.bootstrapper.js.map