"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
const testing_1 = require('@angular/core/testing');
const core_1 = require('@angular/core');
const logger_service_1 = require('../../common/services/logger.service');
const logger_service_mock_1 = require('../../common/services/logger.service.mock');
const bootstrap_1 = require('./bootstrap');
const entityRegistry_1 = require('../../common/registry/entityRegistry');
const service_1 = require('../../common/services/service');
const remoteCli_service_mock_1 = require('../services/remoteCli.service.mock');
const remoteCli_service_1 = require('../services/remoteCli.service');
const abstract_server_mock_1 = require('../servers/abstract.server.mock');
const abstract_server_1 = require('../servers/abstract.server');
const auth_service_mock_1 = require('../services/auth.service.mock');
const auth_service_1 = require('../services/auth.service');
let loggerInstance = new logger_service_mock_1.LoggerMock();
let INIT_SUCCESS = true;
const providers = [
    {
        provide: logger_service_1.Logger,
        useValue: loggerInstance
    },
    { provide: abstract_server_1.Server, useClass: abstract_server_mock_1.ServerMock },
    { provide: remoteCli_service_1.RemoteCli, useClass: remoteCli_service_mock_1.RemoteCliMock },
    { provide: auth_service_1.AuthService, useClass: auth_service_mock_1.AuthServiceMock },
];
let TestService = class TestService extends service_1.AbstractService {
    constructor(logger) {
        super();
        this.logger = logger;
    }
    initialize() {
        this.logger.info('Initializing service');
        return INIT_SUCCESS ? Promise.resolve(this) : Promise.reject(new Error('Service init failed'));
    }
    testLog(message) {
        this.logger.info(message);
    }
};
TestService = __decorate([
    core_1.Injectable(), 
    __metadata('design:paramtypes', [logger_service_1.Logger])
], TestService);
exports.TestService = TestService;
describe('Service Bootstrapper', () => {
    beforeEach(() => {
        testing_1.TestBed.configureTestingModule({ providers });
        entityRegistry_1.EntityRegistry.clearAll();
        entityRegistry_1.EntityRegistry.register('service', TestService);
        INIT_SUCCESS = true;
    });
    it('resolves and initializes service', (done) => {
        const loggerSpy = spyOn(loggerInstance, 'persistLog')
            .and
            .callThrough();
        spyOn(loggerInstance, 'source')
            .and
            .callFake(() => loggerInstance);
        const result = bootstrap_1.bootstrap(undefined, providers)();
        return result.then((res) => {
            const service = res.injector.get(TestService);
            expect(loggerSpy)
                .toHaveBeenCalledWith('info', ['Initializing service']);
            service.testLog('test log');
            expect(loggerSpy)
                .toHaveBeenCalledWith('info', ['test log']);
            done();
        });
    });
    it('aborts bootstrap when a fatal error occurs', (done) => {
        //as there is a fallback to output to log when a fatal bootstrap happens even when mocked,
        //here we spy on the console to suppress the log output
        spyOn(console, 'error');
        spyOn(console, 'log');
        const processExitSpy = spyOn(process, 'exit');
        const loggerSpy = spyOn(loggerInstance, 'persistLog')
            .and
            .callThrough();
        spyOn(loggerInstance, 'source')
            .and
            .callFake(() => loggerInstance);
        INIT_SUCCESS = false;
        const result = bootstrap_1.bootstrap(undefined, providers)();
        return result.then((res) => {
            expect(processExitSpy)
                .toHaveBeenCalledWith(1);
            expect(loggerSpy)
                .toHaveBeenCalledWith('critical', ['Error', 'Service init failed']);
            done();
        });
    });
    it('can provide a new value for a registry instantiated service', (done) => {
        class EnhancedTestService extends TestService {
            initialize() {
                return this.enhancedInit();
            }
            enhancedInit() {
                return Promise.resolve(this);
            }
        }
        let enhanceSpy;
        providers.push({
            provide: TestService,
            deps: [logger_service_1.Logger],
            useFactory: (logger) => {
                const service = new EnhancedTestService(logger);
                enhanceSpy = spyOn(service, 'enhancedInit')
                    .and
                    .callThrough();
                return service;
            },
        });
        const result = bootstrap_1.bootstrap(undefined, providers)();
        return result.then((res) => {
            const service = res.injector.get(TestService);
            expect(service instanceof EnhancedTestService)
                .toBe(true);
            expect(enhanceSpy)
                .toHaveBeenCalled();
            done();
        });
    });
    it('instantiates a service that has no explicit initialize method', (done) => {
        class BasicTestService extends service_1.AbstractService {
            test() {
                return true;
            }
        }
        let initSpy;
        providers.push({
            provide: TestService,
            deps: [],
            useFactory: () => {
                const service = new BasicTestService();
                initSpy = spyOn(service, 'initialize')
                    .and
                    .callThrough();
                return service;
            },
        });
        const result = bootstrap_1.bootstrap(undefined, providers)();
        return result.then((res) => {
            const service = res.injector.get(TestService);
            expect(service instanceof BasicTestService)
                .toBe(true);
            expect(service.test())
                .toEqual(true);
            expect(initSpy)
                .toHaveBeenCalled();
            done();
        });
    });
});
//# sourceMappingURL=services.bootstrapper.spec.js.map