"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
const core_1 = require('@angular/core');
const logger_service_1 = require('../../common/services/logger.service');
const middleware_decorator_1 = require('../middleware/middleware.decorator');
const entityRegistry_1 = require('../../common/registry/entityRegistry');
const exceptions_1 = require('../../common/exceptions/exceptions');
/**
 * Abstract controller that all controllers *must* extend from. The [[ControllerBootstrapper]]
 * relies on the interface provided by this class to invoke registration of routes and middleware
 */
let AbstractController = class AbstractController extends entityRegistry_1.RegistryEntity {
    constructor(logger) {
        super();
        this.logger = logger.source(this.constructor.name);
    }
    /**
     * Register a reference to the current injector, this is not injected directly as there is likely
     * to be a better way to get an injector reference for decorators
     * @see https://github.com/angular/angular/issues/4112#issuecomment-175200243
     * @param injector
     * @returns {AbstractController}
     */
    registerInjector(injector) {
        this.injector = injector;
        return this;
    }
    /**
     * Register an action. This is used by the @Route() decorator, but can also be called directly
     * for custom route registration
     * @param methodSignature
     * @param method
     * @param route
     */
    registerActionMethod(methodSignature, method, route) {
        if (!this.actionMethods) {
            this.actionMethods = new Map();
        }
        const methodDefinition = {
            method,
            route
        };
        this.actionMethods.set(methodSignature, methodDefinition);
    }
    /**
     * Middleware registration. This is used by the @Before & @After decorators to assign middleware
     * to the controller method instance.
     * @param location
     * @param middlewareFactories
     * @param methodSignature
     * @returns void
     */
    static registerMiddleware(location, middlewareFactories, methodSignature) {
        middleware_decorator_1.initializeMiddlewareRegister(this);
        const middleware = this.getMetadata().middleware;
        if (methodSignature) {
            let current = middleware.methods.get(methodSignature);
            if (!current) {
                current = {
                    before: [],
                    after: []
                };
                middleware.methods.set(methodSignature, current);
            }
            current[location].push(...middlewareFactories);
        }
        else {
            middleware.all[location].push(...middlewareFactories);
        }
    }
    /**
     * Register all routes defined in this controller (or any extending instances) with the server
     * @returns {AbstractController}
     */
    registerRoutes(server) {
        this.actionMethods.forEach((methodDefinition, methodSignature) => {
            let callStack = [];
            callStack.push(this[methodSignature]);
            if (this.getMetadata().middleware) {
                const methodMiddlewareFactories = this.getMetadata()
                    .middleware
                    .methods
                    .get(methodSignature);
                //wrap method registered factories with the class defined ones [beforeAll, before, after,
                // afterAll]
                const beforeMiddleware = this.getMetadata()
                    .middleware
                    .all
                    .before
                    .concat(methodMiddlewareFactories.before);
                const afterMiddleware = methodMiddlewareFactories.after.concat(this.getMetadata().middleware.all.after);
                if (methodMiddlewareFactories) {
                    callStack.unshift(...beforeMiddleware.map((middleware) => middleware(this.injector)));
                    callStack.push(...afterMiddleware.map((middleware) => middleware(this.injector)));
                }
            }
            server.register({
                methodName: methodSignature,
                method: methodDefinition.method,
                path: `${process.env.API_BASE}/${this.getMetadata().routeBase}${methodDefinition.route}`,
                callStack: callStack,
                callStackHandler: (request, response) => {
                    return callStack.reduce((current, next) => {
                        return current.then((response) => {
                            return Promise.resolve(next.call(this, request, response));
                        });
                    }, Promise.resolve(response)) //initial value
                        .catch((e) => {
                        if (!(e instanceof exceptions_1.HttpException)) {
                            e = new exceptions_1.InternalServerErrorException(e.message);
                        }
                        response.status(e.getStatusCode());
                        const message = e.getData() || e.toString();
                        response.data({ message });
                        return response;
                    });
                }
            });
            this.logger.debug(`registered ${methodDefinition.method} ${this.getMetadata().routeBase}${methodDefinition.route} to ${this.constructor.name}@${methodSignature}`);
        });
        return this;
    }
};
AbstractController.__metadataDefault = {
    routeBase: '/'
};
AbstractController = __decorate([
    core_1.Injectable(), 
    __metadata('design:paramtypes', [logger_service_1.Logger])
], AbstractController);
exports.AbstractController = AbstractController;
//# sourceMappingURL=abstract.controller.js.map