/// <reference types="node" />
/**
 * @module server
 */
/** End Typedoc Module Declaration */
import { IncomingMessage } from 'http';
/**
 * Request class that is passed into all middleware and controller methods to extract data from the
 * client request.
 */
export declare class Request {
    protected raw: IncomingMessage;
    protected paramsMap: Map<string, string>;
    protected headersMap: Map<string, string>;
    /** @event Emitted when request body data is read */
    static EVENT_DATA: string;
    /** @event Emitted when request body read is completed */
    static EVENT_END: string;
    /** @event Emitted when response body read is completed */
    static EVENT_ERROR: string;
    constructor(raw?: IncomingMessage, paramsMap?: Map<string, string>, headersMap?: Map<string, string>);
    /**
     * Get the raw IncomingMessage
     * @returns {IncomingMessage}
     */
    getRaw(): IncomingMessage;
    /**
     * Get all headers as a Map
     * @returns {Map<string, string>}
     */
    headers(): Map<string, string>;
    /**
     * Get all route paramaters as a Map
     * @returns {Map<string, string>}
     */
    params(): Map<string, string>;
    /**
     * Helper function for converting Dictionary to Map
     * @param dictionary
     * @returns {Map<any, any>}
     */
    static extractMapFromDictionary<K, V>(dictionary: Object): Map<K, V>;
    /**
     * Extract the passed body object from the raw request
     * @returns {Promise<T>}
     */
    getPayload<T>(): Promise<T>;
}
