"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
/**
 * @module server
 */
/** End Typedoc Module Declaration */
const core_1 = require('@angular/core');
const abstract_controller_1 = require('./abstract.controller');
const logger_service_1 = require('../../common/services/logger.service');
const route_decorator_1 = require('./route.decorator');
const response_1 = require('./response');
const request_1 = require('./request');
const store_1 = require('../../common/stores/store');
const exceptions_1 = require('../../common/exceptions/exceptions');
/**
 * Provides resource controller that all controllers that interact RESTfully with ModelStores
 * should extend from.
 */
let ResourceController = class ResourceController extends abstract_controller_1.AbstractController {
    constructor(logger, modelStore) {
        super(logger);
        this.modelStore = modelStore;
    }
    /**
     * Get one entity
     * @param request
     * @param response
     * @returns {any}
     */
    getOne(request, response) {
        return __awaiter(this, void 0, void 0, function* () {
            const model = yield this.modelStore.findOne(request.params().get('id'));
            return response.data(model);
        });
    }
    /**
     * Get many entities
     * @param request
     * @param response
     * @returns {any}
     */
    getMany(request, response) {
        return __awaiter(this, void 0, void 0, function* () {
            const collection = yield this.modelStore.findMany();
            return response.data(collection);
        });
    }
    /**
     * Process and persist an entity
     * @param request
     * @param response
     */
    putOne(request, response) {
        return this.savePayload(request, response);
    }
    /**
     * Process and update entity, skipping validation of any missing properties
     * @param request
     * @param response
     * @returns {Promise<Response>}
     */
    patchOne(request, response) {
        return this.savePayload(request, response, true, {
            skipMissingProperties: true,
        });
    }
    /**
     * Delete the payload model from the model store
     * @param request
     * @param response
     * @returns {Promise<Response>}
     */
    deleteOne(request, response) {
        return __awaiter(this, void 0, void 0, function* () {
            const model = yield this.modelStore.hydrate(yield request.getPayload());
            yield this.modelStore.deleteOne(model);
            return response.data(model);
        });
    }
    /**
     * Persist the request payload with the model store with optional validator options
     * @param request
     * @param response
     * @param validatorOptions
     * @param checkExists
     * @returns {Promise<Response>}
     */
    savePayload(request, response, checkExists = false, validatorOptions = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const model = yield this.modelStore.hydrate(yield request.getPayload());
            if (checkExists) {
                const exists = yield this.modelStore.hasOne(model);
                if (!exists) {
                    throw new exceptions_1.NotFoundException(`Model with id [${model.getIdentifier()}] does not exist`);
                }
            }
            yield this.modelStore.validate(model, validatorOptions);
            yield this.modelStore.saveOne(model);
            return response.data(model);
        });
    }
};
__decorate([
    route_decorator_1.Route('GET', '/:id'), 
    __metadata('design:type', Function), 
    __metadata('design:paramtypes', [request_1.Request, response_1.Response]), 
    __metadata('design:returntype', Promise)
], ResourceController.prototype, "getOne", null);
__decorate([
    route_decorator_1.Route('GET', '/'), 
    __metadata('design:type', Function), 
    __metadata('design:paramtypes', [request_1.Request, response_1.Response]), 
    __metadata('design:returntype', Promise)
], ResourceController.prototype, "getMany", null);
__decorate([
    route_decorator_1.Route('PUT', '/:id'), 
    __metadata('design:type', Function), 
    __metadata('design:paramtypes', [request_1.Request, response_1.Response]), 
    __metadata('design:returntype', Promise)
], ResourceController.prototype, "putOne", null);
__decorate([
    route_decorator_1.Route('PATCH', '/:id'), 
    __metadata('design:type', Function), 
    __metadata('design:paramtypes', [request_1.Request, response_1.Response]), 
    __metadata('design:returntype', Promise)
], ResourceController.prototype, "patchOne", null);
__decorate([
    route_decorator_1.Route('DELETE', '/:id'), 
    __metadata('design:type', Function), 
    __metadata('design:paramtypes', [request_1.Request, response_1.Response]), 
    __metadata('design:returntype', Promise)
], ResourceController.prototype, "deleteOne", null);
ResourceController = __decorate([
    core_1.Injectable(), 
    __metadata('design:paramtypes', [logger_service_1.Logger, store_1.AbstractStore])
], ResourceController);
exports.ResourceController = ResourceController;
//# sourceMappingURL=resource.controller.js.map