"use strict";
/**
 * @module server
 */
/** End Typedoc Module Declaration */
const abstract_server_1 = require('./servers/abstract.server');
const database_service_1 = require('./services/database.service');
const remoteCli_service_1 = require('./services/remoteCli.service');
const logger_service_1 = require('../common/services/logger.service');
const consoleLogger_service_1 = require('../common/services/consoleLogger.service');
const debugLog_middleware_1 = require('./middleware/debugLog.middleware');
const express_server_1 = require('./servers/express.server');
const dotenv = require('dotenv');
const path = require('path');
const _ = require('lodash');
// Load .env variables into process.env.*
dotenv.config({
    path: path.resolve(process.cwd(), '.env')
});
// Strip the prefix PUBLIC_ from any exported .env vars
process.env = _.mapKeys(process.env, (value, key) => key.replace(/^PUBLIC_/, ''));
/**
 * Core providers, can be overridden by implementing app with it's own provider definitions
 * @type {ProviderDefinition[]}
 */
exports.CORE_PROVIDERS = [
    database_service_1.Database,
    remoteCli_service_1.RemoteCli,
    debugLog_middleware_1.DebugLogMiddleware,
    // {provide: Server, useClass: HapiServer},
    { provide: abstract_server_1.Server, useClass: express_server_1.ExpressServer },
    { provide: logger_service_1.Logger, useClass: consoleLogger_service_1.ConsoleLogger },
];
//# sourceMappingURL=main.js.map