"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
/**
 * @module server
 */
/** End Typedoc Module Declaration */
const core_1 = require('@angular/core');
const logger_service_1 = require('../../common/services/logger.service');
/**
 * Basic debug logger middleware See [[debugLog]] for usage
 */
let DebugLogMiddleware = class DebugLogMiddleware {
    constructor(loggerBase) {
        this.logger = loggerBase.source('debugLog');
    }
    /**
     * Creates the debugLog middleware with binding to current class for access to logger
     * @param messages
     * @returns {any}
     */
    middlewareFactory(messages) {
        return function debugLog(request, response) {
            this.logger.debug(...messages);
            return response;
        }.bind(this);
    }
};
DebugLogMiddleware = __decorate([
    core_1.Injectable(), 
    __metadata('design:paramtypes', [logger_service_1.Logger])
], DebugLogMiddleware);
exports.DebugLogMiddleware = DebugLogMiddleware;
/**
 * Logs messages to the Logger implementation when middleware is invoked
 * Passes through any responses
 *
 * Example usage:
 * ```typescript
 *  @Injectable()
 *  @Controller()
 * class ExampleController extends AbstractController {
 *
 *  constructor(server: Server, logger: Logger) {
 *    super(server, logger);
 *  }
 *
 *  @Route('GET', '/test')
 *  @Before(debugLog('test log input'))
 *  public testMethod(request: Request, response: Response): Response {
 *    return response;
 *  }
 *
 * }
 * ```
 * When `GET /test` is called, "test log input" will be logged before the testMethod is invoked
 *
 * @param messages
 * @returns {function(ReflectiveInjector): Middleware}
 */
function debugLog(...messages) {
    return (injector) => {
        return injector.get(DebugLogMiddleware).middlewareFactory(messages);
    };
}
exports.debugLog = debugLog;
//# sourceMappingURL=debugLog.middleware.js.map