"use strict";
const metadata_1 = require('../../common/metadata/metadata');
/**
 * Decorator for assigning before middleware method in a controller
 * @returns {function(any, string, TypedPropertyDescriptor<T>): undefined}
 * @constructor
 * @param middlewareFactories
 */
function Before(...middlewareFactories) {
    return function (target, propertyKey, descriptor) {
        target.constructor.registerMiddleware('before', middlewareFactories, propertyKey);
    };
}
exports.Before = Before;
/**
 * Decorator for assigning after middleware method in a controller
 * @returns {function(any, string, TypedPropertyDescriptor<T>): undefined}
 * @constructor
 * @param middlewareFactories
 */
function After(...middlewareFactories) {
    return function (target, propertyKey, descriptor) {
        target.constructor.registerMiddleware('after', middlewareFactories, propertyKey);
    };
}
exports.After = After;
/**
 * Initializes the `registeredMiddleware` property on the controller with empty stores
 * @param target
 */
function initializeMiddlewareRegister(target) {
    metadata_1.initializeMetadata(target);
    if (!target.__metadata.middleware) {
        target.__metadata.middleware = {
            methods: new Map(),
            all: {
                before: [],
                after: []
            }
        };
    }
}
exports.initializeMiddlewareRegister = initializeMiddlewareRegister;
/**
 * Decorator for assigning before middleware to all methods in a controller
 * @param middlewareFactories
 * @returns {function(AbstractController): void}
 * @constructor
 */
function BeforeAll(...middlewareFactories) {
    return function (target) {
        target.registerMiddleware('before', middlewareFactories);
    };
}
exports.BeforeAll = BeforeAll;
/**
 * Decorator for assigning after middleware to all methods in a controller
 * @param middlewareFactories
 * @returns {function(AbstractController): void}
 * @constructor
 */
function AfterAll(...middlewareFactories) {
    return function (target) {
        target.registerMiddleware('after', middlewareFactories);
    };
}
exports.AfterAll = AfterAll;
//# sourceMappingURL=middleware.decorator.js.map