/**
 * @module server
 */
/** End Typedoc Module Declaration */
import { Database } from '../services/database.service';
import { Logger } from '../../common/services/logger.service';
/**
 * Root class that all implementations of seeders *must* extend. Provides common interface for
 * bootstrapper to handle seeding
 */
export declare abstract class AbstractMigration {
    protected database: Database;
    protected logger: Logger;
    constructor(loggerBase: Logger, database: Database);
    /**
     * Starts the migration. Returns promise so bootstrapper can wait until finished before starting
     * the next migration
     */
    abstract migrate(): Promise<void>;
    /**
     * Reverts the migration. Returns promise so bootstrapper can wait until finished before rolling
     * back the next migration
     */
    abstract rollback(): Promise<void>;
}
