"use strict";
/**
 * @module server
 */
/** End Typedoc Module Declaration */
const decorators_1 = require('../../common/registry/decorators');
/**
 *  @Controller class decorator for registering class with the [[EntityRegistry]]
 *
 * Example:
 * ```typescript
 *  import { Controller } from '@zerothstack/core/common';
 *  import { AbstractController } from '@zerothstack/core/server';
 *
 *  @Controller()
 *  export class ExampleController extends AbstractController {}
 * ```
 * @param metadata
 * @returns {ClassDecorator}
 * @constructor
 */
function Controller(metadata) {
    return decorators_1.entityRegistryFunction('controller', metadata);
}
exports.Controller = Controller;
/**
 * @module common
 */
/** End Typedoc Module Declaration */
/**
 *  @Seeder class decorator for registering class with the [[EntityRegistry]]
 *
 * Example:
 * ```typescript
 *  import { Seeder } from '@zerothstack/core/common';
 *  import { AbstractSeeder } from '@zerothstack/core/server';
 *
 *  @Seeder()
 *  export class ExampleSeeder extends AbstractSeeder {}
 * ```
 * @returns {ClassDecorator}
 * @constructor
 */
function Seeder() {
    return decorators_1.entityRegistryFunction('seeder');
}
exports.Seeder = Seeder;
/**
 *  @Migration class decorator for registering class with the [[EntityRegistry]]
 *
 * Example:
 * ```typescript
 *  import { Migration } from '@zerothstack/core/common';
 *  import { AbstractMigration } from '@zerothstack/core/server';
 *
 *  @Migration()
 *  export class ExampleMigration extends AbstractMigration {}
 * ```
 * @returns {ClassDecorator}
 * @constructor
 */
function Migration() {
    return decorators_1.entityRegistryFunction('migration');
}
exports.Migration = Migration;
//# sourceMappingURL=decorators.js.map