"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
/**
 * @module server
 */
/** End Typedoc Module Declaration */
const core_1 = require('@angular/core');
const remoteCli_service_1 = require('../services/remoteCli.service');
const logger_service_1 = require('../../common/services/logger.service');
const response_1 = require('../controllers/response');
/**
 * Root class that all implementations of server *must* extends. Provides common interface for
 * bootstrapper to handle server startup without caring about underlying implementation
 */
let Server = class Server {
    constructor(loggerBase, remoteCli) {
        /** All Configured routes */
        this.configuredRoutes = [];
        this.logger = loggerBase.source('server');
        this.host = process.env.APP_HOST || null; //usually should be null to default binding to localhost
        this.port = process.env.APP_PORT || 3000;
        this.initialize();
        remoteCli.start(3001);
    }
    /**
     * Registration function for routes
     * @param config
     */
    register(config) {
        this.configuredRoutes.push(config);
        return this.registerRouteWithEngine(config);
    }
    ;
    /**
     * Kicks off the server
     */
    start() {
        this.registerStaticLoader(process.env.WEB_ROOT);
        return this.startEngine();
    }
    ;
    /**
     * Retrieve the base instance of require('http').Server
     * @returns {HttpServer}
     */
    getHttpServer() {
        return this.httpServer;
    }
    /**
     * Get the host name (for logging)
     * @returns {string}
     */
    getHost() {
        return `http://${this.host || '(localhost)'}:${this.port}`;
    }
    /**
     * Retrieve all configured routes
     * @returns {RouteConfig[]}
     */
    getRoutes() {
        return this.configuredRoutes;
    }
    /**
     * Get the default response object
     * @returns {Response}
     */
    getDefaultResponse() {
        return new response_1.Response()
            .header('X-Powered-By', `Zeroth<Angular,${this.constructor.name.replace('Server', '')}>`);
    }
};
Server = __decorate([
    core_1.Injectable(), 
    __metadata('design:paramtypes', [logger_service_1.Logger, remoteCli_service_1.RemoteCli])
], Server);
exports.Server = Server;
//# sourceMappingURL=abstract.server.js.map