"use strict";
const abstract_server_1 = require('./abstract.server');
const remoteCli_service_1 = require('../services/remoteCli.service');
const logger_service_1 = require('../../common/services/logger.service');
const testing_1 = require('@angular/core/testing');
const logger_service_mock_1 = require('../../common/services/logger.service.mock');
const remoteCli_service_mock_1 = require('../services/remoteCli.service.mock');
const response_1 = require('../controllers/response');
const auth_service_1 = require('../services/auth.service');
const auth_service_mock_1 = require('../services/auth.service.mock');
const abstract_server_mock_1 = require('./abstract.server.mock');
describe('Server', () => {
    const providers = [
        { provide: abstract_server_1.Server, useClass: abstract_server_mock_1.ServerMock },
        { provide: logger_service_1.Logger, useClass: logger_service_mock_1.LoggerMock },
        { provide: remoteCli_service_1.RemoteCli, useClass: remoteCli_service_mock_1.RemoteCliMock },
        { provide: auth_service_1.AuthService, useClass: auth_service_mock_1.AuthServiceMock },
    ];
    let cliSpy;
    beforeEach(() => {
        testing_1.TestBed.configureTestingModule({ providers });
        spyOn(abstract_server_mock_1.ServerMock.prototype, 'initialize')
            .and
            .callThrough();
        cliSpy = spyOn(remoteCli_service_mock_1.RemoteCliMock.prototype, 'start');
    });
    it('initializes the server with port and host', testing_1.inject([abstract_server_1.Server], (server) => {
        expect(server.initialize)
            .toHaveBeenCalled();
        expect(cliSpy)
            .toHaveBeenCalledWith(3001);
        expect(server.getHost())
            .toEqual('http://(localhost):3000');
    }));
    it('returns the engine', testing_1.inject([abstract_server_1.Server], (server) => {
        expect(server.getEngine())
            .toBe(undefined);
    }));
    it('returns the inner http server instance', testing_1.inject([abstract_server_1.Server], (server) => {
        expect(server.getHttpServer())
            .toBe(undefined);
    }));
    it('registers routes', testing_1.inject([abstract_server_1.Server], (server) => {
        const routeConfig = {
            path: '/test',
            methodName: 'test',
            method: 'GET',
            callStack: [],
            callStackHandler: null
        };
        let spy = spyOn(server, 'registerRouteWithEngine');
        server.register(routeConfig);
        expect(spy)
            .toHaveBeenCalledWith(routeConfig);
        expect(server.getRoutes())
            .toEqual([routeConfig]);
    }));
    it('starts the server running and returns promise', testing_1.async(testing_1.inject([abstract_server_1.Server], (server) => {
        let spy = spyOn(server, 'start')
            .and
            .callThrough();
        let response = server.start();
        expect(spy)
            .toHaveBeenCalled();
        return response.then((onStart) => {
            expect(onStart)
                .toEqual(server);
        });
    })));
    it('gets default response object from server', testing_1.inject([abstract_server_1.Server], (server) => {
        const response = server.getDefaultResponse();
        expect(response instanceof response_1.Response)
            .toBe(true);
    }));
});
//# sourceMappingURL=abstract.server.spec.js.map