"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
/**
 * @module server
 */
/** End Typedoc Module Declaration */
const core_1 = require('@angular/core');
const abstract_server_1 = require('./abstract.server');
const remoteCli_service_1 = require('../services/remoteCli.service');
const logger_service_1 = require('../../common/services/logger.service');
const request_1 = require('../controllers/request');
const express = require('express');
const http = require('http');
let ExpressServer = class ExpressServer extends abstract_server_1.Server {
    constructor(logger, remoteCli) {
        super(logger, remoteCli);
    }
    /**
     * @inheritdoc
     * @returns {Application}
     */
    getEngine() {
        return this.app;
    }
    /**
     * @inheritdoc
     * @returns {Express}
     */
    initialize() {
        this.app = express();
        this.httpServer = http.createServer((this.app));
        return this;
    }
    /**
     * @inheritdoc
     * @returns {any}
     * @param routeConfig
     */
    registerRouteWithEngine(routeConfig) {
        this.app[routeConfig.method.toLowerCase()](routeConfig.path, (req, res) => {
            let request = new request_1.Request(req, request_1.Request.extractMapFromDictionary(req.params), request_1.Request.extractMapFromDictionary(req.headers));
            let response = this.getDefaultResponse();
            return routeConfig.callStackHandler(request, response)
                .then((response) => {
                return this.send(response, res);
            })
                .catch((err) => this.sendErr(err, res));
        });
        return this;
    }
    /**
     * @inheritDoc
     */
    registerStaticLoader(webroot) {
        if (webroot) {
            this.app.use(express.static(webroot, { index: ['index.html'] }));
        }
        //@todo resolve how to load webpacked modules with angular/universal
        // const ngApp = (req:ExpressRequest, res:ExpressResponse) => {
        //   let baseUrl = '/';
        //   let url = req.originalUrl || '/';
        //
        //   let config: ExpressEngineConfig = {
        //     directives: frontendComponents,//[ App ],
        //
        //     // dependencies shared among all requests to server
        //     platformProviders: [
        //       {provide: ORIGIN_URL, useValue: this.getHost()},
        //       {provide: BASE_URL, useValue: baseUrl},
        //     ],
        //
        //     // dependencies re-created for each request
        //     providers: [
        //       {provide: REQUEST_URL, useValue: url},
        //       // provideRouter(routes),
        //       NODE_LOCATION_PROVIDERS,
        //       NODE_HTTP_PROVIDERS,
        //     ],
        //
        //     // if true, server will wait for all async to resolve before returning response
        //     async: true,
        //
        //     // if you want preboot, you need to set selector for the app root
        //     // you can also include various preboot options here (explained in separate document)
        //     preboot: false // { appRoot: 'app' }
        //   };
        //
        //   res.render('index', config);
        // };
        //
        // this.app.engine('.html', expressEngine);
        // this.app.set('views', process.env.WEB_ROOT);
        // this.app.set('view engine', 'html');
        // this.app.use(express.static(process.env.WEB_ROOT, {index: false}));
        // this.app.get('/', ngApp);
        return this;
    }
    /**
     * @inheritdoc
     * @returns {Promise<ExpressServer>}
     */
    startEngine() {
        return new Promise((resolve, reject) => {
            this.httpServer.listen(this.port, this.host, resolve);
        })
            .then(() => this);
    }
    /**
     * Send the response
     * @param response
     * @param res
     */
    send(response, res) {
        res.status(response.statusCode);
        for (var [key, value] of response.headers.entries()) {
            res.header(key, value);
        }
        res.send(response.body);
    }
    /**
     * Send the error response
     * @param err
     * @param res
     */
    sendErr(err, res) {
        //make sure the status is of error type
        if (res.statusCode < 400) {
            res.status(500);
        }
        res.send(err);
    }
};
ExpressServer = __decorate([
    core_1.Injectable(), 
    __metadata('design:paramtypes', [logger_service_1.Logger, remoteCli_service_1.RemoteCli])
], ExpressServer);
exports.ExpressServer = ExpressServer;
//# sourceMappingURL=express.server.js.map