import { Server as Hapi, IReply, Response as HapiResponse } from 'hapi';
import { Server, RouteConfig } from './abstract.server';
import { RemoteCli } from '../services/remoteCli.service';
import { Logger } from '../../common/services/logger.service';
import { Response } from '../controllers/response';
export declare class HapiServer extends Server {
    private engine;
    constructor(logger: Logger, remoteCli: RemoteCli);
    /**
     * @inheritdoc
     * @returns {Hapi}
     */
    getEngine(): Hapi;
    /**
     * @inheritdoc
     * @returns {HapiServer}
     */
    protected initialize(): this;
    /**
     * @inheritdoc
     * @returns {any}
     * @param routeConfig
     */
    protected registerRouteWithEngine(routeConfig: RouteConfig): this;
    /**
     * Send the response
     * @param response
     * @param reply
     * @return {HapiResponse}
     */
    protected send(response: Response, reply: IReply): HapiResponse;
    /**
     * Send the error response
     * @param err
     * @param reply
     * @return {HapiResponse}
     */
    protected sendErr(err: Error, reply: IReply): HapiResponse;
    /**
     * @inheritdoc
     */
    registerStaticLoader(webroot?: string): this;
    /**
     * @inheritdoc
     * @returns {Promise<HapiServer>}
     */
    startEngine(): Promise<this>;
}
