"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
/**
 * @module server
 */
/** End Typedoc Module Declaration */
const core_1 = require('@angular/core');
const hapi_1 = require('hapi');
const abstract_server_1 = require('./abstract.server');
const remoteCli_service_1 = require('../services/remoteCli.service');
const logger_service_1 = require('../../common/services/logger.service');
const response_1 = require('../controllers/response');
const request_1 = require('../controllers/request');
const exceptions_1 = require('../../common/exceptions/exceptions');
let HapiServer = class HapiServer extends abstract_server_1.Server {
    constructor(logger, remoteCli) {
        super(logger, remoteCli);
    }
    /**
     * @inheritdoc
     * @returns {Hapi}
     */
    getEngine() {
        return this.engine;
    }
    /**
     * @inheritdoc
     * @returns {HapiServer}
     */
    initialize() {
        this.engine = new hapi_1.Server();
        this.engine.connection({
            host: this.host,
            port: this.port
        });
        this.httpServer = this.engine.listener;
        return this;
    }
    /**
     * @inheritdoc
     * @returns {any}
     * @param routeConfig
     */
    registerRouteWithEngine(routeConfig) {
        if (/[\*\?]/.test(routeConfig.path)) {
            throw new Error('Hapi syntax for optional or multi-segment parameters is not supported');
        }
        const config = {
            //re-map /path/{param} to /path/{param} (the inverse if needed later is
            // .replace(/{([-_a-zA-Z0-9]+).*?}/g, ':$1')
            path: routeConfig.path.replace(/:(.+?)(\/|$)/g, "{$1}$2"),
            method: routeConfig.method,
            handler: (req, reply) => {
                let request = new request_1.Request(req.raw.req, //typings are incorrect, type should be IncomingMessage
                request_1.Request.extractMapFromDictionary(req.params), request_1.Request.extractMapFromDictionary(req.headers));
                let response = this.getDefaultResponse();
                return routeConfig.callStackHandler(request, response)
                    .then((response) => this.send(response, reply))
                    .catch((err) => this.sendErr(err, reply));
            }
        };
        this.engine.route(config);
        return this;
    }
    /**
     * Send the response
     * @param response
     * @param reply
     * @return {HapiResponse}
     */
    send(response, reply) {
        const res = reply(response.body);
        res.code(response.statusCode);
        for (var [key, value] of response.headers.entries()) {
            res.header(key, value);
        }
        return res;
    }
    /**
     * Send the error response
     * @param err
     * @param reply
     * @return {HapiResponse}
     */
    sendErr(err, reply) {
        const errorResponse = new response_1.Response().data(err);
        const res = this.send(errorResponse, reply);
        //make sure the status is of error type
        if (res.statusCode < 400) {
            res.code(500);
        }
        return res;
    }
    /**
     * @inheritdoc
     */
    registerStaticLoader(webroot) {
        if (webroot) {
            throw new exceptions_1.NotImplementedException('Static file listing is not implemented for hapi');
        }
        return this;
    }
    /**
     * @inheritdoc
     * @returns {Promise<HapiServer>}
     */
    startEngine() {
        return new Promise((resolve, reject) => {
            this.engine.start((err) => {
                if (err) {
                    return reject(err);
                }
                return resolve();
            });
        })
            .then(() => this);
    }
};
HapiServer = __decorate([
    core_1.Injectable(), 
    __metadata('design:paramtypes', [logger_service_1.Logger, remoteCli_service_1.RemoteCli])
], HapiServer);
exports.HapiServer = HapiServer;
//# sourceMappingURL=hapi.server.js.map