"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
const logger_service_1 = require('../../common/services/logger.service');
const database_service_1 = require('./database.service');
const testing_1 = require('@angular/core/testing');
const typeorm = require('typeorm');
const core_1 = require('@angular/core');
const remoteCli_service_1 = require('./remoteCli.service');
const logger_service_mock_1 = require('../../common/services/logger.service.mock');
const remoteCli_service_mock_1 = require('./remoteCli.service.mock');
const entityRegistry_1 = require('../../common/registry/entityRegistry');
const SQL = require('sql-template-strings');
const auth_service_mock_1 = require('./auth.service.mock');
const auth_service_1 = require('./auth.service');
let ExampleUtil = class ExampleUtil {
    constructor(database, logger) {
        this.database = database;
        this.logger = logger;
    }
    flagLongUsernames(role, length) {
        let queryRunner;
        return this.database.getDriver()
            .then((driver) => driver.createQueryRunner())
            .then((qr) => {
            queryRunner = qr;
            return qr.beginTransaction();
        })
            .then(() => this.database.query(database_service_1.Database.prepare `UPDATE users SET flagged = LENGTH(username) > ${length} WHERE role = ${role}`))
            .then(() => queryRunner.commitTransaction())
            .catch(() => queryRunner.rollbackTransaction());
    }
};
ExampleUtil = __decorate([
    core_1.Injectable(), 
    __metadata('design:paramtypes', [database_service_1.Database, logger_service_1.Logger])
], ExampleUtil);
describe('Database', () => {
    let createConnectionSpy;
    const connectionSpy = {};
    const providers = [
        { provide: logger_service_1.Logger, useClass: logger_service_mock_1.LoggerMock },
        { provide: remoteCli_service_1.RemoteCli, useClass: remoteCli_service_mock_1.RemoteCliMock },
        { provide: auth_service_1.AuthService, useClass: auth_service_mock_1.AuthServiceMock },
        database_service_1.Database,
        ExampleUtil
    ];
    const envMap = {
        DB_DRIVER: 'driver',
        DB_HOST: 'host',
        DB_PORT: 'port',
        DB_USERNAME: 'username',
        DB_PASSWORD: 'password',
        DB_DATABASE: 'database',
    };
    const connectionConfigFixture = {
        name: 'default',
        driver: {
            type: envMap.DB_DRIVER,
            host: envMap.DB_HOST,
            port: envMap.DB_PORT,
            username: envMap.DB_USERNAME,
            password: envMap.DB_PASSWORD,
            database: envMap.DB_DATABASE,
            usePool: false,
        },
        autoSchemaSync: false,
        logging: {
            logger: jasmine.any(Function),
            logQueries: true,
        },
        entities: [],
    };
    beforeEach(() => {
        testing_1.TestBed.configureTestingModule({ providers });
        entityRegistry_1.EntityRegistry.clearAll();
        Object.assign(process.env, envMap);
        createConnectionSpy = spyOn(typeorm, 'createConnection')
            .and
            .callFake(() => {
            return Promise.resolve(connectionSpy);
        });
    });
    afterEach(() => {
        database_service_1.Database.clearConnections();
    });
    it('initializes database with connection', testing_1.async(testing_1.inject([database_service_1.Database], (database) => {
        expect(createConnectionSpy)
            .toHaveBeenCalledWith(connectionConfigFixture);
        return database.getConnection()
            .then((conn) => {
            expect(conn)
                .toEqual(connectionSpy);
        });
    })));
    it('retrieves an instance of the connection', testing_1.async(testing_1.inject([database_service_1.Database], (database) => {
        return database.getConnection()
            .then((conn) => {
            expect(conn)
                .toEqual(connectionSpy);
        });
    })));
    it('interfaces the typeorm logger function with the logger function', testing_1.inject([database_service_1.Database, logger_service_1.Logger], (database, logger) => {
        const logFunctionSpy = jasmine.createSpy('logFunction');
        database_service_1.Database.connect('default', logFunctionSpy);
        expect(createConnectionSpy)
            .toHaveBeenCalledWith(connectionConfigFixture);
        const logFunction = createConnectionSpy.calls.mostRecent().args[0].logging.logger;
        logFunction('log', 'log with level log');
        expect(logFunctionSpy)
            .toHaveBeenCalledWith('info', 'log with level log');
        logFunction('warn', 'log with level warn');
        expect(logFunctionSpy)
            .toHaveBeenCalledWith('warning', 'log with level warn');
        logFunction('error', 'log with level error');
        expect(logFunctionSpy)
            .toHaveBeenCalledWith('error', 'log with level error');
    }));
    describe('Connection error', () => {
        it('rejects connection if connection fails', testing_1.async(testing_1.inject([database_service_1.Database], (database) => {
            // clear cached default connection
            database_service_1.Database.clearConnections();
            // force failure
            createConnectionSpy.and.callFake(() => {
                return Promise.reject(new Error('Connection error'));
            });
            return database.initialize()
                .catch((error) => {
                expect(error.message)
                    .toEqual('Connection error');
            });
        })));
    });
    it('executes a raw query on the database', testing_1.async(testing_1.inject([database_service_1.Database], (database) => {
        const resultMock = [{ foo: 'bar' }];
        const queryRunnerMock = {
            query: jasmine.createSpy('query')
                .and
                .returnValue(resultMock)
        };
        connectionSpy.driver = {
            createQueryRunner: jasmine.createSpy('createQueryRunner')
                .and
                .returnValue(Promise.resolve(queryRunnerMock))
        };
        const queryString = 'SELECT * FROM table';
        return database.query(queryString)
            .then((result) => {
            expect(queryRunnerMock.query)
                .toHaveBeenCalledWith(queryString);
            expect(result)
                .toEqual(resultMock);
        });
    })));
    describe('Complex queries', () => {
        it('runs complex queries with transactions and prepared statements', testing_1.async(testing_1.inject([ExampleUtil, database_service_1.Database], (util, database) => {
            const resultMock = [{ foo: 'bar' }];
            const queryRunnerMock = {
                beginTransaction: jasmine.createSpy('beginTransaction').and.returnValue(Promise.resolve()),
                commitTransaction: jasmine.createSpy('commitTransaction')
                    .and
                    .returnValue(Promise.resolve()),
                query: jasmine.createSpy('query')
                    .and
                    .returnValue(resultMock)
            };
            connectionSpy.driver = {
                createQueryRunner: jasmine.createSpy('createQueryRunner')
                    .and
                    .returnValue(Promise.resolve(queryRunnerMock))
            };
            return util.flagLongUsernames('admin', 5)
                .then(() => {
                expect(queryRunnerMock.beginTransaction).toHaveBeenCalled();
                expect(queryRunnerMock.query)
                    .toHaveBeenCalledWith(SQL `UPDATE users SET flagged = LENGTH(username) > ${5} WHERE role = ${'admin'}`);
                expect(queryRunnerMock.commitTransaction).toHaveBeenCalled();
            });
        })));
        it('runs complex queries with exceptions rolling back transactions', testing_1.async(testing_1.inject([ExampleUtil, database_service_1.Database], (util, database) => {
            const queryRunnerMock = {
                beginTransaction: jasmine.createSpy('beginTransaction').and.returnValue(Promise.resolve()),
                commitTransaction: jasmine.createSpy('commitTransaction')
                    .and
                    .returnValue(Promise.resolve()),
                rollbackTransaction: jasmine.createSpy('rollbackTransaction')
                    .and
                    .returnValue(Promise.resolve()),
                query: jasmine.createSpy('query').and.returnValue(Promise.reject(new Error('DB Error'))),
            };
            connectionSpy.driver = {
                createQueryRunner: jasmine.createSpy('createQueryRunner')
                    .and
                    .returnValue(Promise.resolve(queryRunnerMock))
            };
            return util.flagLongUsernames('admin', 5)
                .then(() => {
                expect(queryRunnerMock.beginTransaction).toHaveBeenCalled();
                expect(queryRunnerMock.query).toHaveBeenCalled();
                expect(queryRunnerMock.commitTransaction).not.toHaveBeenCalled();
                expect(queryRunnerMock.rollbackTransaction).toHaveBeenCalled();
            });
        })));
    });
});
//# sourceMappingURL=database.service.spec.js.map