"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
/**
 * @module server
 */
/** End Typedoc Module Declaration */
const banner_1 = require('../../common/util/banner');
const chalk = require('chalk');
const path = require('path');
exports.jwtAuthStrategyFactory = (remoteCliContext) => {
    return function (vantage, options) {
        return function (args, cb) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    remoteCliContext.logger.silly.debug('Passed client arguments: ', args);
                    const token = args.client.jwt;
                    let keyPath = args.client.publicKeyPath;
                    if (!token) {
                        return cb("JWT was not passed in connection request", false);
                    }
                    if (process.env.PATH_ROOT) {
                        keyPath = path.resolve(process.env.PATH_ROOT, keyPath);
                    }
                    remoteCliContext.logger.info(`Authenticating JSON web token against public key [${keyPath}]`);
                    const payload = yield remoteCliContext.authService.verify(token, keyPath);
                    remoteCliContext.logger.info(`${payload.username} has been authenticated with token`)
                        .debug('Token:', token);
                    let displayBanner = `Hi ${payload.username}, Welcome to Zeroth runtime cli.`;
                    if (args.client.columns >= 80) {
                        displayBanner = banner_1.bannerBg(undefined, token);
                    }
                    this.log(chalk.grey(`You were authenticated with a JSON Web token verified against the public key at ${keyPath}`));
                    this.log(displayBanner);
                    this.log(` Type 'help' for a list of available commands`);
                    return cb(null, true);
                }
                catch (e) {
                    remoteCliContext.logger.error('Authentication error', e.message).debug(e.stack);
                    cb(e.message, false);
                }
            });
        };
    };
};
//# sourceMappingURL=jwtAuthStrategy.js.map