"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
/**
 * @module server
 */
/** End Typedoc Module Declaration */
const core_1 = require('@angular/core');
const logger_service_1 = require('../../common/services/logger.service');
const abstract_server_1 = require('../servers/abstract.server');
const chalk = require('chalk');
const Vantage = require('vantage');
const decorators_1 = require('../../common/registry/decorators');
const service_1 = require('../../common/services/service');
const auth_service_1 = require('./auth.service');
const jwtAuthStrategy_1 = require('./jwtAuthStrategy');
const table = require('table').default;
/**
 * Class allows developers to register custom commands that can be remote executed in a
 * shell environment. Useful for things like migrations and debugging.
 */
let RemoteCli = class RemoteCli extends service_1.AbstractService {
    constructor(loggerBase, injector, authService) {
        super();
        this.injector = injector;
        this.authService = authService;
        this.logger = loggerBase.source('remote-cli');
    }
    /**
     * Initialize the vantage client
     * @returns {RemoteCli}
     */
    initialize() {
        this.vantage = new Vantage();
        this.vantage.delimiter(chalk.magenta('zeroth-runtime~$'));
        this.registerAuthenticationStrategy();
        this.logger.debug('Remote cli initialized');
        return this.registerCommands();
    }
    /**
     * Registers the pre-defined commands
     */
    registerCommands() {
        let remoteCli = this;
        this.vantage.command('routes')
            .description('outputs route table')
            .action(function (args, callback) {
            remoteCli.logger.info('CLI session retrieving routes');
            let server = remoteCli.injector.get(abstract_server_1.Server);
            const routeTable = server.getRoutes()
                .map((route) => {
                // @todo break into newlines when 'table' supports it
                const stack = route.callStack.map((handler) => handler.name);
                return [route.method, route.path, stack];
            });
            routeTable.unshift(['Method', 'Path', 'Stack'].map((s) => chalk.blue(s)));
            let table = remoteCli.makeTable(routeTable);
            this.log('\n' + table);
            callback();
        });
        return this;
    }
    /**
     * Starts the Vantage server. This is done on start of the server so debugging can start
     * immediately
     * @param port
     * @param callback
     */
    start(port, callback) {
        if (!callback) {
            callback = (socket) => {
                this.logger.info(`Accepted a connection from [${socket.conn.remoteAddress}]`);
            };
        }
        this.logger.debug('Auth function', this.vantage._authFn);
        this.vantage.listen(port, callback);
        this.logger.info(`Vantage server started on ${port}`);
        return this;
    }
    /**
     * Constructs table string for output to the cli
     * @see https://github.com/gajus/table
     * @param data
     * @param config
     * @returns {string}
     */
    makeTable(data, config) {
        return table(data, config);
    }
    /**
     * Register the authentication strategy with vantage
     */
    registerAuthenticationStrategy() {
        //when more auth strategies exist this should be refactored to injected class
        const strategy = jwtAuthStrategy_1.jwtAuthStrategyFactory(this);
        this.vantage.auth(strategy);
        this.logger.debug('Registered vantage authentication strategy');
    }
};
RemoteCli = __decorate([
    core_1.Injectable(),
    decorators_1.Service(), 
    __metadata('design:paramtypes', [logger_service_1.Logger, core_1.Injector, auth_service_1.AuthService])
], RemoteCli);
exports.RemoteCli = RemoteCli;
//# sourceMappingURL=remoteCli.service.js.map