/**
 * @module server
 */
/** End Typedoc Module Declaration */
import { Injector } from '@angular/core';
import { AbstractModel, ModelStatic, identifier } from '../../common/models/model';
import { Database } from '../services/database.service';
import { Logger } from '../../common/services/logger.service';
import { AbstractStore, Query } from '../../common/stores/store';
import { Collection } from '../../common/models/collection';
import { Repository } from 'typeorm';
/**
 * Database store should be extended with a specific implementation for a model. Interacts with
 * TypeORM's repository to handle CRUD with the database
 */
export declare abstract class DatabaseStore<T extends AbstractModel> extends AbstractStore<T> {
    protected database: Database;
    protected loggerBase: Logger;
    /**
     * The TypeORM repository instance
     */
    protected repositoryPromise: Promise<Repository<T>>;
    /**
     * Logger for the class, initialized with source
     */
    protected logger: Logger;
    constructor(modelStatic: ModelStatic<T>, injector: Injector, database: Database, loggerBase: Logger);
    /**
     * Retrieve the TypeORM repository for this store's modelStatic.
     * This promise is cached so the same repository instance is always returned
     * @returns {Promise<Repository<T>>}
     */
    getRepository(): Promise<Repository<T>>;
    /**
     * @inheritdoc
     */
    initialized(): Promise<this>;
    /**
     * @inheritdoc
     */
    findOne(id: identifier): Promise<T>;
    /**
     * @inheritdoc
     */
    findMany(query?: Query): Promise<Collection<T>>;
    /**
     * @inheritdoc
     */
    saveOne(model: T): Promise<T>;
    /**
     * @inheritdoc
     */
    deleteOne(model: T): Promise<T>;
    /**
     * @inheritdoc
     */
    hasOne(model: T): Promise<boolean>;
}
