"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
/**
 * @module server
 */
/** End Typedoc Module Declaration */
const core_1 = require('@angular/core');
const database_service_1 = require('../services/database.service');
const logger_service_1 = require('../../common/services/logger.service');
const store_1 = require('../../common/stores/store');
const collection_1 = require('../../common/models/collection');
const exceptions_1 = require('../../common/exceptions/exceptions');
/**
 * Database store should be extended with a specific implementation for a model. Interacts with
 * TypeORM's repository to handle CRUD with the database
 */
let DatabaseStore = class DatabaseStore extends store_1.AbstractStore {
    constructor(modelStatic, injector, database, loggerBase) {
        super(modelStatic, injector);
        this.database = database;
        this.loggerBase = loggerBase;
        this.logger = loggerBase.source('DB Store');
    }
    /**
     * Retrieve the TypeORM repository for this store's modelStatic.
     * This promise is cached so the same repository instance is always returned
     * @returns {Promise<Repository<T>>}
     */
    getRepository() {
        if (!this.repositoryPromise) {
            this.repositoryPromise = this.database.getConnection()
                .then((connection) => connection.getRepository(this.modelStatic));
        }
        return this.repositoryPromise;
    }
    /**
     * @inheritdoc
     */
    initialized() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.getRepository();
            return this;
        });
    }
    /**
     * @inheritdoc
     */
    findOne(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const repo = yield this.getRepository();
            const model = yield repo.findOneById(id);
            if (!model) {
                throw new exceptions_1.NotFoundException(`${this.modelStatic.name} not found with id [${id}]`);
            }
            return model;
        });
    }
    /**
     * @inheritdoc
     */
    findMany(query) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const repo = yield this.getRepository();
                const entityArray = yield repo.find({});
                if (!entityArray.length) {
                    throw new exceptions_1.NotFoundException(`No ${this.modelStatic.name} found with query params [${JSON.stringify(query)}]`);
                }
                return new collection_1.Collection(entityArray);
            }
            catch (e) {
                this.logger.error(e);
                throw e;
            }
        });
    }
    /**
     * @inheritdoc
     */
    saveOne(model) {
        return __awaiter(this, void 0, void 0, function* () {
            const repo = yield this.getRepository();
            return repo.persist(model);
        });
    }
    /**
     * @inheritdoc
     */
    deleteOne(model) {
        return __awaiter(this, void 0, void 0, function* () {
            const repo = yield this.getRepository();
            return repo.remove(model);
        });
    }
    /**
     * @inheritdoc
     */
    hasOne(model) {
        return __awaiter(this, void 0, void 0, function* () {
            const repo = yield this.getRepository();
            try {
                yield repo.findOneById(model.getIdentifier());
                return true;
            }
            catch (e) {
                return false;
            }
        });
    }
};
DatabaseStore = __decorate([
    core_1.Injectable(), 
    __metadata('design:paramtypes', [Object, core_1.Injector, database_service_1.Database, logger_service_1.Logger])
], DatabaseStore);
exports.DatabaseStore = DatabaseStore;
//# sourceMappingURL=db.store.js.map