"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
const logger_service_1 = require('../../common/services/logger.service');
const core_1 = require('@angular/core');
const testing_1 = require('@angular/core/testing');
const logger_service_mock_1 = require('../../common/services/logger.service.mock');
const database_service_1 = require('../services/database.service');
const db_store_1 = require('./db.store');
const model_1 = require('../../common/models/model');
const primary_decorator_1 = require('../../common/models/types/primary.decorator');
const storedProperty_decorator_1 = require('../../common/models/types/storedProperty.decorator');
const collection_1 = require('../../common/models/collection');
const database_service_mock_1 = require('../services/database.service.mock');
const exceptions_1 = require('../../common/exceptions/exceptions');
class TestModel extends model_1.AbstractModel {
}
__decorate([
    primary_decorator_1.Primary(), 
    __metadata('design:type', Number)
], TestModel.prototype, "id", void 0);
__decorate([
    storedProperty_decorator_1.StoredProperty(), 
    __metadata('design:type', String)
], TestModel.prototype, "name", void 0);
let TestDatabaseStore = class TestDatabaseStore extends db_store_1.DatabaseStore {
    constructor(injector, database, loggerBase) {
        super(TestModel, injector, database, loggerBase);
    }
};
TestDatabaseStore = __decorate([
    core_1.Injectable(), 
    __metadata('design:paramtypes', [core_1.Injector, database_service_1.Database, logger_service_1.Logger])
], TestDatabaseStore);
describe('Database Store', () => {
    const dbConnectionSpy = jasmine.createSpyObj('dbConnection', ['getRepository']);
    const repositorySpy = jasmine.createSpyObj('repository', ['findOneById', 'find', 'persist', 'remove']);
    dbConnectionSpy.getRepository.and.returnValue(repositorySpy);
    const providers = [
        TestDatabaseStore,
        { provide: logger_service_1.Logger, useClass: logger_service_mock_1.LoggerMock },
        { provide: database_service_1.Database, useClass: database_service_mock_1.DatabaseMock },
    ];
    beforeEach(() => {
        testing_1.TestBed.configureTestingModule({ providers });
    });
    it('initializes the store with a reference to the static model', testing_1.inject([TestDatabaseStore], (store) => {
        expect(store.modelStatic).toEqual(TestModel);
    }));
    it('retrieves a reference to the orm repository', testing_1.async(testing_1.inject([TestDatabaseStore, database_service_1.Database], (store, db) => {
        database_service_1.Database.connections.set('default', dbConnectionSpy);
        return store.getRepository()
            .then((repo) => {
            expect(repo)
                .toEqual(repositorySpy);
            expect(dbConnectionSpy.getRepository)
                .toHaveBeenCalledTimes(1);
            //verify repo is cached by re-calling and asserting no more calls to getRepository
            return store.getRepository()
                .then(() => {
                expect(repo)
                    .toEqual(repositorySpy);
                expect(dbConnectionSpy.getRepository)
                    .toHaveBeenCalledTimes(1);
            });
        });
    })));
    it('provides initialized method so callees can defer actions', testing_1.async(testing_1.inject([TestDatabaseStore, database_service_1.Database], (store, db) => {
        database_service_1.Database.connections.set('default', dbConnectionSpy);
        return store.initialized()
            .then((res) => {
            expect(res)
                .toEqual(store);
        });
    })));
    it('retrieves a single entity from the orm', testing_1.async(testing_1.inject([TestDatabaseStore, database_service_1.Database], (store, db) => {
        database_service_1.Database.connections.set('default', dbConnectionSpy);
        const testModelFixture = new TestModel({ id: 10 });
        repositorySpy.findOneById.and.returnValue(Promise.resolve(testModelFixture));
        return store.findOne(testModelFixture.id)
            .then((res) => {
            expect(repositorySpy.findOneById)
                .toHaveBeenCalledWith(testModelFixture.id);
            expect(res)
                .toEqual(testModelFixture);
        });
    })));
    it('rejects retrieval with not found exception when there is no model', testing_1.async(testing_1.inject([TestDatabaseStore, database_service_1.Database], (store, db) => {
        database_service_1.Database.connections.set('default', dbConnectionSpy);
        repositorySpy.findOneById.and.returnValue(Promise.resolve(null));
        return store.findOne(123)
            .catch((err) => {
            expect(err instanceof exceptions_1.NotFoundException)
                .toBe(true);
            expect(err.message)
                .toEqual('TestModel not found with id [123]');
        });
    })));
    it('deletes a single entity from the orm', testing_1.async(testing_1.inject([TestDatabaseStore, database_service_1.Database], (store, db) => {
        database_service_1.Database.connections.set('default', dbConnectionSpy);
        const testModelFixture = new TestModel({ id: 10 });
        repositorySpy.remove.and.returnValue(Promise.resolve(testModelFixture));
        return store.deleteOne(testModelFixture)
            .then((res) => {
            expect(repositorySpy.remove)
                .toHaveBeenCalledWith(testModelFixture);
            expect(res)
                .toEqual(testModelFixture);
        });
    })));
    it('retrieves collection of entities from the orm', testing_1.async(testing_1.inject([TestDatabaseStore, database_service_1.Database], (store, db) => {
        database_service_1.Database.connections.set('default', dbConnectionSpy);
        const testModelsFixture = [new TestModel({ id: 12 })];
        repositorySpy.find.and.returnValue(Promise.resolve(testModelsFixture));
        return store.findMany()
            .then((res) => {
            expect(repositorySpy.find)
                .toHaveBeenCalledWith({});
            expect(res instanceof collection_1.Collection)
                .toBe(true);
            expect(res[0])
                .toEqual(testModelsFixture[0]);
        });
    })));
    it('rejects retrieval with not found exception when there is no models', testing_1.async(testing_1.inject([TestDatabaseStore, database_service_1.Database], (store, db) => {
        database_service_1.Database.connections.set('default', dbConnectionSpy);
        repositorySpy.find.and.returnValue(Promise.resolve([]));
        return store.findMany()
            .catch((err) => {
            expect(repositorySpy.find)
                .toHaveBeenCalledWith({});
            expect(err instanceof exceptions_1.NotFoundException)
                .toBe(true);
            expect(err.message)
                .toEqual('No TestModel found with query params [undefined]');
        });
    })));
    it('persists a single entity to the orm', testing_1.async(testing_1.inject([TestDatabaseStore, database_service_1.Database], (store, db) => {
        database_service_1.Database.connections.set('default', dbConnectionSpy);
        const testModelFixture = new TestModel({ id: 10 });
        repositorySpy.persist.and.returnValue(Promise.resolve(testModelFixture));
        return store.saveOne(testModelFixture)
            .then((res) => {
            expect(repositorySpy.persist)
                .toHaveBeenCalledWith(testModelFixture);
            expect(res)
                .toEqual(testModelFixture);
        });
    })));
    it('checks if entity exists in the orm', testing_1.async(testing_1.inject([TestDatabaseStore, database_service_1.Database], (store, db) => {
        database_service_1.Database.connections.set('default', dbConnectionSpy);
        const testModelFixture = new TestModel({ id: 10 });
        repositorySpy.findOneById.and.returnValue(Promise.resolve(testModelFixture));
        return store.hasOne(testModelFixture)
            .then((res) => {
            expect(res)
                .toBe(true);
            repositorySpy.findOneById.and.returnValue(Promise.reject(new Error('not found')));
            return store.hasOne(testModelFixture);
        })
            .then((res) => {
            expect(res)
                .toBe(false);
        });
    })));
});
//# sourceMappingURL=db.store.spec.js.map